"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = fetch;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _analytics() {
  const data = require("../../../analytics/analytics");

  _analytics = function () {
    return data;
  };

  return data;
}

function _loader() {
  const data = _interopRequireDefault(require("../../../cli/loader"));

  _loader = function () {
    return data;
  };

  return data;
}

function _consumer() {
  const data = require("../../../consumer");

  _consumer = function () {
    return data;
  };

  return data;
}

function _importComponents() {
  const data = _interopRequireDefault(require("../../../consumer/component-ops/import-components"));

  _importComponents = function () {
    return data;
  };

  return data;
}

function _generalError() {
  const data = _interopRequireDefault(require("../../../error/general-error"));

  _generalError = function () {
    return data;
  };

  return data;
}

function _laneId() {
  const data = require("../../../lane-id/lane-id");

  _laneId = function () {
    return data;
  };

  return data;
}

function fetch(_x, _x2, _x3) {
  return _fetch.apply(this, arguments);
}

function _fetch() {
  _fetch = (0, _bluebird().coroutine)(function* (ids, lanes, components) {
    if (!lanes && !components) {
      throw new (_generalError().default)(`please provide the type of objects you would like to pull, the options are --components and --lanes`);
    }

    _loader().default.start('fetching objects...');

    const consumer = yield (0, _consumer().loadConsumer)();
    const importOptions = {
      ids,
      objectsOnly: true,
      verbose: false,
      withEnvironments: false,
      writePackageJson: false,
      writeConfig: false,
      writeDists: false,
      override: false,
      installNpmPackages: false
    };
    const importComponents = new (_importComponents().default)(consumer, importOptions);

    if (lanes) {
      const laneIds = yield getLaneIds();
      importOptions.lanes = {
        laneIds
      };
    }

    const {
      dependencies,
      envComponents,
      importDetails
    } = yield importComponents.importComponents();
    const bitIds = dependencies.map(_ramda().default.path(['component', 'id']));

    _analytics().Analytics.setExtraData('num_components', bitIds.length);

    yield consumer.onDestroy();
    return {
      dependencies,
      envComponents,
      importDetails
    };

    function getLaneIds() {
      return _getLaneIds.apply(this, arguments);
    }

    function _getLaneIds() {
      _getLaneIds = (0, _bluebird().coroutine)(function* () {
        if (ids.length) {
          return ids.map(id => {
            const trackLane = consumer.scope.lanes.getRemoteTrackedDataByLocalLane(id);
            if (trackLane) return _laneId().RemoteLaneId.from(trackLane.remoteLane, trackLane.remoteScope); // assuming it's a remote

            return _laneId().RemoteLaneId.parse(id);
          });
        }

        return consumer.scope.objects.remoteLanes.getAllRemoteLaneIds();
      });
      return _getLaneIds.apply(this, arguments);
    }
  });
  return _fetch.apply(this, arguments);
}