"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = switchAction;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _loader() {
  const data = _interopRequireDefault(require("../../../cli/loader"));

  _loader = function () {
    return data;
  };

  return data;
}

function _loaderMessages() {
  const data = require("../../../cli/loader/loader-messages");

  _loaderMessages = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

function _consumer() {
  const data = require("../../../consumer");

  _consumer = function () {
    return data;
  };

  return data;
}

function _createLane() {
  const data = _interopRequireDefault(require("../../../consumer/lanes/create-lane"));

  _createLane = function () {
    return data;
  };

  return data;
}

function _switchLanes() {
  const data = _interopRequireDefault(require("../../../consumer/lanes/switch-lanes"));

  _switchLanes = function () {
    return data;
  };

  return data;
}

function _generalError() {
  const data = _interopRequireDefault(require("../../../error/general-error"));

  _generalError = function () {
    return data;
  };

  return data;
}

function _laneId() {
  const data = require("../../../lane-id/lane-id");

  _laneId = function () {
    return data;
  };

  return data;
}

function _scopeComponentsImporter() {
  const data = _interopRequireDefault(require("../../../scope/component-ops/scope-components-importer"));

  _scopeComponentsImporter = function () {
    return data;
  };

  return data;
}

function switchAction(_x, _x2) {
  return _switchAction.apply(this, arguments);
}

function _switchAction() {
  _switchAction = (0, _bluebird().coroutine)(function* (switchProps, checkoutProps) {
    _loader().default.start(_loaderMessages().BEFORE_CHECKOUT);

    const consumer = yield (0, _consumer().loadConsumer)();
    let results;

    if (switchProps.create) {
      yield (0, _createLane().default)(consumer, switchProps.laneName);
      consumer.scope.lanes.setCurrentLane(switchProps.laneName);
      results = {
        added: switchProps.laneName
      };
    } else {
      yield populateSwitchProps(consumer, switchProps);
      results = yield (0, _switchLanes().default)(consumer, switchProps, checkoutProps);
    }

    yield consumer.onDestroy();
    return results;
  });
  return _switchAction.apply(this, arguments);
}

function populateSwitchProps(_x3, _x4) {
  return _populateSwitchProps.apply(this, arguments);
}

function _populateSwitchProps() {
  _populateSwitchProps = (0, _bluebird().coroutine)(function* (consumer, switchProps) {
    const lanes = yield consumer.scope.listLanes();
    const {
      laneName,
      remoteScope
    } = switchProps;

    if (remoteScope) {
      yield populatePropsAccordingToRemoteLane();
    } else {
      populatePropsAccordingToLocalLane();
    }

    function populatePropsAccordingToRemoteLane() {
      return _populatePropsAccordingToRemoteLane.apply(this, arguments);
    }

    function _populatePropsAccordingToRemoteLane() {
      _populatePropsAccordingToRemoteLane = (0, _bluebird().coroutine)(function* () {
        // fetch the remote to update all heads
        const localTrackedLane = consumer.scope.lanes.getLocalTrackedLaneByRemoteName(laneName, remoteScope);
        switchProps.localLaneName = switchProps.newLaneName || localTrackedLane || laneName;

        if (consumer.getCurrentLaneId().name === switchProps.localLaneName) {
          throw new (_generalError().default)(`already checked out to "${switchProps.localLaneName}"`);
        }

        const scopeComponentImporter = _scopeComponentsImporter().default.getInstance(consumer.scope);

        const remoteLaneObjects = yield scopeComponentImporter.importFromLanes([_laneId().RemoteLaneId.from(laneName, remoteScope)]);
        const remoteLaneComponents = remoteLaneObjects[0].components;
        const laneExistsLocally = lanes.find(l => l.name === switchProps.localLaneName);

        if (laneExistsLocally) {
          throw new (_generalError().default)(`unable to checkout to a remote lane ${remoteScope}/${laneName}.
    the local lane ${switchProps.localLaneName} already exists, please switch to the local lane first by omitting --remote flag
    then run "bit merge" to merge the remote lane into the local lane`);
        }

        switchProps.ids = remoteLaneComponents.map(l => l.id.changeVersion(l.head.toString()));
        switchProps.remoteLaneScope = remoteScope;
        switchProps.remoteLaneName = laneName;
        switchProps.remoteLaneComponents = remoteLaneComponents;
        switchProps.localTrackedLane = localTrackedLane || undefined;
      });
      return _populatePropsAccordingToRemoteLane.apply(this, arguments);
    }

    function populatePropsAccordingToLocalLane() {
      switchProps.localLaneName = laneName;

      if (consumer.getCurrentLaneId().name === laneName) {
        throw new (_generalError().default)(`already checked out to "${laneName}"`);
      }

      if (laneName === _constants().DEFAULT_LANE) {
        switchProps.ids = consumer.bitMap.getAuthoredAndImportedBitIdsOfDefaultLane();
        return;
      }

      const localLane = lanes.find(lane => lane.name === laneName);

      if (!localLane) {
        throw new (_generalError().default)(`unable to find a local lane "${laneName}", to create a new lane please use --create flag`);
      }

      switchProps.ids = localLane.components.map(c => c.id.changeVersion(c.head.toString()));
    }
  });
  return _populateSwitchProps.apply(this, arguments);
}