"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.string.replace");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _fsExtra() {
  const data = _interopRequireDefault(require("fs-extra"));

  _fsExtra = function () {
    return data;
  };

  return data;
}

function path() {
  const data = _interopRequireWildcard(require("path"));

  path = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

function _validationError() {
  const data = _interopRequireDefault(require("../../error/validation-error"));

  _validationError = function () {
    return data;
  };

  return data;
}

function _laneId() {
  const data = require("../../lane-id/lane-id");

  _laneId = function () {
    return data;
  };

  return data;
}

function _logger() {
  const data = _interopRequireDefault(require("../../logger/logger"));

  _logger = function () {
    return data;
  };

  return data;
}

function _utils() {
  const data = require("../../utils");

  _utils = function () {
    return data;
  };

  return data;
}

function _addComponents() {
  const data = _interopRequireDefault(require("../component-ops/add-components"));

  _addComponents = function () {
    return data;
  };

  return data;
}

function _exceptions() {
  const data = require("../component-ops/add-components/exceptions");

  _exceptions = function () {
    return data;
  };

  return data;
}

function _componentNotFoundInPath() {
  const data = _interopRequireDefault(require("../component/exceptions/component-not-found-in-path"));

  _componentNotFoundInPath = function () {
    return data;
  };

  return data;
}

function _outsideRootDir() {
  const data = _interopRequireDefault(require("./exceptions/outside-root-dir"));

  _outsideRootDir = function () {
    return data;
  };

  return data;
}

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2().default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

class ComponentMap {
  // always set for IMPORTED and NESTED.
  // reason why trackDir and not re-use rootDir is because using rootDir requires all paths to be
  // relative to rootDir for consistency, then, when saving into the model changing them back to
  // be relative to consumer-root. (we can't save in the model relative to rootDir, otherwise the
  // dependencies paths won't work).
  // relevant for AUTHORED only when a component was added as a directory, used for tracking changes in that dir
  // directory shared among a component and its dependencies by the original author. Relevant for IMPORTED only
  // a wrapper directory needed when a user adds a package.json file to the component root so then it won't collide with Bit generated one
  // wether the compiler / tester are detached from the workspace global configuration
  // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
  // relevant for authored components only, it helps finding out whether a component has a scope
  // whether a component is available only on lanes and not on master
  // save component versions per lanes if they're different than the id
  // if a component was created on another lane, it might not be available on the current lane
  // for soft-tag (harmony only), this data is used in the CI to persist
  constructor({
    id,
    files,
    mainFile,
    rootDir,
    trackDir,
    origin,
    originallySharedDir,
    wrapDir,
    onLanesOnly,
    lanes,
    defaultVersion,
    isAvailableOnCurrentLane,
    nextVersion
  }) {
    (0, _defineProperty2().default)(this, "id", void 0);
    (0, _defineProperty2().default)(this, "files", void 0);
    (0, _defineProperty2().default)(this, "mainFile", void 0);
    (0, _defineProperty2().default)(this, "rootDir", void 0);
    (0, _defineProperty2().default)(this, "trackDir", void 0);
    (0, _defineProperty2().default)(this, "origin", void 0);
    (0, _defineProperty2().default)(this, "originallySharedDir", void 0);
    (0, _defineProperty2().default)(this, "wrapDir", void 0);
    (0, _defineProperty2().default)(this, "markBitMapChangedCb", void 0);
    (0, _defineProperty2().default)(this, "exported", void 0);
    (0, _defineProperty2().default)(this, "onLanesOnly", false);
    (0, _defineProperty2().default)(this, "lanes", void 0);
    (0, _defineProperty2().default)(this, "defaultVersion", void 0);
    (0, _defineProperty2().default)(this, "isAvailableOnCurrentLane", true);
    (0, _defineProperty2().default)(this, "nextVersion", void 0);
    this.id = id;
    this.files = files;
    this.mainFile = mainFile;
    this.rootDir = rootDir;
    this.trackDir = trackDir;
    this.origin = origin;
    this.originallySharedDir = originallySharedDir;
    this.wrapDir = wrapDir;
    this.onLanesOnly = onLanesOnly;
    this.lanes = lanes || [];
    this.defaultVersion = defaultVersion;
    this.isAvailableOnCurrentLane = isAvailableOnCurrentLane;
    this.nextVersion = nextVersion;
  }

  static fromJson(componentMapObj) {
    const componentMapParams = _objectSpread(_objectSpread({}, componentMapObj), {}, {
      lanes: componentMapObj.lanes ? componentMapObj.lanes.map(lane => ({
        remoteLane: _laneId().RemoteLaneId.parse(lane.remoteLane),
        version: lane.version
      })) : []
    });

    return new ComponentMap(componentMapParams);
  }

  toPlainObject() {
    let res = {
      files: this.files.map(file => (0, _utils().sortObject)(file)),
      mainFile: this.mainFile,
      rootDir: this.rootDir,
      trackDir: this.trackDir,
      origin: this.origin,
      originallySharedDir: this.originallySharedDir,
      wrapDir: this.wrapDir,
      exported: this.exported,
      onLanesOnly: this.onLanesOnly || null,
      // if false, change to null so it won't be written
      lanes: this.lanes.map(l => ({
        remoteLane: l.remoteLane.toString(),
        version: l.version
      })),
      nextVersion: this.nextVersion
    };

    const notNil = val => {
      return !_ramda().default.isNil(val);
    };

    res = _ramda().default.filter(notNil, res);
    return res;
  }

  static getPathWithoutRootDir(rootDir, filePath) {
    const newPath = (0, _utils().pathRelativeLinux)(rootDir, filePath);

    if (newPath.startsWith('..')) {
      // this is forbidden for security reasons. Allowing files to be written outside the components directory may
      // result in overriding OS files.
      throw new (_outsideRootDir().default)(filePath, rootDir);
    }

    return newPath;
  }

  static changeFilesPathAccordingToItsRootDir(existingRootDir, files) {
    const changes = [];
    files.forEach(file => {
      const newPath = this.getPathWithoutRootDir(existingRootDir, file.relativePath); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
      // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

      changes.push({
        from: file.relativePath,
        to: newPath
      });
      file.relativePath = newPath;
    });
    return changes;
  }

  setMarkAsChangedCb(markAsChangedBinded) {
    this.markBitMapChangedCb = markAsChangedBinded;
  }

  _findFile(fileName) {
    return this.files.find(file => {
      const filePath = this.rootDir ? (0, _utils().pathJoinLinux)(this.rootDir, file.relativePath) : file.relativePath;
      return filePath === fileName;
    });
  }

  changeRootDirAndUpdateFilesAccordingly(newRootDir) {
    if (this.rootDir === newRootDir) return;
    this.files.forEach(file => {
      const filePathRelativeToConsumer = this.rootDir ? (0, _utils().pathJoinLinux)(this.rootDir, file.relativePath) : file.relativePath;
      const newPath = ComponentMap.getPathWithoutRootDir(newRootDir, filePathRelativeToConsumer);
      if (this.mainFile === file.relativePath) this.mainFile = newPath;
      file.relativePath = newPath;
    });
    this.rootDir = newRootDir;
    this.trackDir = undefined; // if there is trackDir, it's not needed anymore.
  }

  addRootDirToDistributedFiles(rootDir) {
    this.files.forEach(file => {
      file.relativePath = file.name;
    });
    this.rootDir = (0, _utils().pathNormalizeToLinux)(rootDir);
    this.mainFile = path().basename(this.mainFile);
    this.validate();
  }

  updateFileLocation(fileFrom, fileTo) {
    fileFrom = (0, _utils().pathNormalizeToLinux)(fileFrom);
    fileTo = (0, _utils().pathNormalizeToLinux)(fileTo);

    const currentFile = this._findFile(fileFrom);

    const changes = [];

    if (currentFile) {
      const rootDir = this.rootDir;
      const newLocation = rootDir ? ComponentMap.getPathWithoutRootDir(rootDir, fileTo) : fileTo;

      _logger().default.debug(`updating file location from ${currentFile.relativePath} to ${newLocation}`);

      if (this.mainFile === currentFile.relativePath) this.mainFile = newLocation; // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
      // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

      changes.push({
        from: currentFile.relativePath,
        to: newLocation
      });
      currentFile.relativePath = newLocation;
      currentFile.name = path().basename(newLocation);
    }

    this.validate();
    return changes;
  }

  updateDirLocation(dirFrom, dirTo) {
    dirFrom = (0, _utils().pathNormalizeToLinux)(dirFrom);
    dirTo = (0, _utils().pathNormalizeToLinux)(dirTo);
    const changes = [];

    if (this.rootDir && this.rootDir.startsWith(dirFrom)) {
      const rootDir = this.rootDir;
      const newRootDir = rootDir.replace(dirFrom, dirTo);
      const newRootDirNormalized = (0, _utils().pathNormalizeToLinux)(newRootDir); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
      // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

      changes.push({
        from: rootDir,
        to: newRootDirNormalized
      });

      _logger().default.debug(`updating rootDir location from ${rootDir} to ${newRootDirNormalized}`);

      this.rootDir = newRootDirNormalized;
      return changes;
    }

    this.files.forEach(file => {
      const filePath = this.rootDir ? path().join(this.rootDir, file.relativePath) : file.relativePath;

      if (filePath.startsWith(dirFrom)) {
        const fileTo = filePath.replace(dirFrom, dirTo);
        const newLocation = this.rootDir ? ComponentMap.getPathWithoutRootDir(this.rootDir, fileTo) : fileTo;

        _logger().default.debug(`updating file location from ${file.relativePath} to ${newLocation}`);

        if (this.mainFile === file.relativePath) this.mainFile = newLocation; // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
        // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

        changes.push({
          from: file.relativePath,
          to: newLocation
        });
        file.relativePath = newLocation;
      }
    });

    if (this.origin === _constants().COMPONENT_ORIGINS.AUTHORED && this.trackDir && this.trackDir === dirFrom) {
      this.trackDir = dirTo;
    }

    this.validate();
    return changes;
  }

  getFilesRelativeToConsumer() {
    return this.files.map(file => {
      return this.rootDir ? (0, _utils().pathJoinLinux)(this.rootDir, file.relativePath) : file.relativePath;
    });
  }

  getAllFilesPaths() {
    return this.files.map(file => file.relativePath);
  }

  getFilesGroupedByBeingTests() {
    const allFiles = [];
    const nonTestsFiles = [];
    const testsFiles = [];
    this.files.forEach(file => {
      // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
      allFiles.push(file.relativePath); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

      if (file.test) testsFiles.push(file.relativePath); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
      else nonTestsFiles.push(file.relativePath);
    });
    return {
      allFiles,
      nonTestsFiles,
      testsFiles
    };
  }
  /**
   * if one of the added files is outside of the trackDir, remove the trackDir attribute
   */


  removeTrackDirIfNeeded() {
    if (!this.trackDir) return;

    if (this.origin !== _constants().COMPONENT_ORIGINS.AUTHORED) {
      this.trackDir = undefined;
      return;
    }

    for (const file of this.files) {
      if (!file.relativePath.startsWith(this.trackDir)) {
        this.trackDir = undefined;
        return;
      }
    }
  }
  /**
   * directory to track for changes (such as files added/renamed)
   */


  getTrackDir() {
    if (this.origin === _constants().COMPONENT_ORIGINS.AUTHORED) return this.rootDir || this.trackDir;

    if (this.origin === _constants().COMPONENT_ORIGINS.IMPORTED) {
      return this.wrapDir ? (0, _utils().pathJoinLinux)(this.rootDir, this.wrapDir) : this.rootDir;
    } // DO NOT track nested components!


    return undefined;
  }
  /**
   * this.rootDir is not defined for author. instead, the current workspace is the rootDir
   * also, for imported environments (compiler/tester) components the rootDir is empty
   */


  getRootDir() {
    return this.rootDir || '.';
  }

  hasRootDir() {
    return Boolean(this.rootDir && this.rootDir !== '.');
  }
  /**
   * directory of the component (root / track)
   * for legacy (bit.json) workspace, it can be undefined for authored when individual files were added
   */


  getComponentDir() {
    if (this.origin === _constants().COMPONENT_ORIGINS.AUTHORED) return this.rootDir || this.trackDir;
    return this.rootDir;
  }

  doesAuthorHaveRootDir() {
    return Boolean(this.origin === _constants().COMPONENT_ORIGINS.AUTHORED && this.rootDir);
  }
  /**
   * this.id.version should indicate the currently used version, regardless of the lane.
   * on the filesystem, id.version is saved according to the master, so it needs to be changed.
   * @param currentRemote
   * @param currentLaneIds
   */


  updatePerLane(currentRemote, currentLaneIds) {
    this.isAvailableOnCurrentLane = undefined;

    const replaceVersion = version => {
      this.defaultVersion = this.id.version;
      this.id = this.id.changeVersion(version);
      this.isAvailableOnCurrentLane = true;
    };

    const localBitId = currentLaneIds ? currentLaneIds.searchWithoutVersion(this.id) : null;

    if (localBitId) {
      replaceVersion(localBitId.version);
    } else if (currentRemote) {
      const remoteExist = this.lanes.find(lane => lane.remoteLane.isEqual(currentRemote));

      if (remoteExist) {
        replaceVersion(remoteExist.version);
      }
    }

    if (typeof this.isAvailableOnCurrentLane === 'undefined') {
      // either, it's the default lane. or, it's a lane and the component is not part of the lane
      this.isAvailableOnCurrentLane = !this.onLanesOnly;
    }
  }

  addLane(remoteLaneId, version) {
    const existing = this.lanes.find(l => l.remoteLane.isEqual(remoteLaneId));
    if (existing) existing.version = version;
    this.lanes.push({
      remoteLane: remoteLaneId,
      version
    });
  }
  /**
   * in case new files were created in the track-dir directory, add them to the component-map
   * so then they'll be tracked by bitmap
   */


  trackDirectoryChanges(consumer, id) {
    var _this = this;

    return (0, _bluebird().coroutine)(function* () {
      const trackDir = _this.getTrackDir();

      if (trackDir) {
        const trackDirAbsolute = path().join(consumer.getPath(), trackDir);
        const trackDirRelative = path().relative(process.cwd(), trackDirAbsolute);
        if (!_fsExtra().default.existsSync(trackDirAbsolute)) throw new (_componentNotFoundInPath().default)(trackDirRelative);
        const addParams = {
          componentPaths: [trackDirRelative || '.'],
          id: id.toString(),
          override: false,
          // this makes sure to not override existing files of componentMap
          trackDirFeature: true,
          origin: _this.origin
        };
        const numOfFilesBefore = _this.files.length;
        const addContext = {
          consumer
        };
        const addComponents = new (_addComponents().default)(addContext, addParams);

        try {
          yield addComponents.add();
        } catch (err) {
          if (err instanceof _exceptions().NoFiles || err instanceof _exceptions().EmptyDirectory) {// it might happen that a component is imported and current .gitignore configuration
            // are effectively removing all files from bitmap. we should ignore the error in that
            // case
          } else {
            throw err;
          }
        }

        if (_this.files.length > numOfFilesBefore) {
          _logger().default.info(`new file(s) have been added to .bitmap for ${id.toString()}`);

          consumer.bitMap.hasChanged = true;
        }
      }
    })();
  }

  updateNextVersion(nextVersion) {
    this.nextVersion = nextVersion;
    this.validate();
  }

  clearNextVersion() {
    delete this.nextVersion;
  }

  removeFiles(files) {
    const relativePaths = files.map(file => file.relativePath);
    this.files = this.files.reduce((accumulator, file) => {
      // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
      return relativePaths.includes(file.relativePath) ? accumulator : accumulator.concat(file);
    }, []);
    this.validate();
  }

  sort() {
    this.files = _ramda().default.sortBy(_ramda().default.prop('relativePath'), this.files);
  }

  clone() {
    // @ts-ignore - there is some issue with the config dir type
    return new ComponentMap(this);
  }

  validate() {
    const errorMessage = `failed adding or updating a ${_constants().BIT_MAP} record of ${this.id.toString()}.`;
    if (!this.mainFile) throw new (_validationError().default)(`${errorMessage} mainFile attribute is missing`);

    if (!(0, _utils().isValidPath)(this.mainFile)) {
      throw new (_validationError().default)(`${errorMessage} mainFile attribute ${this.mainFile} is invalid`);
    } // if it's an environment component (such as compiler) the rootDir is empty


    if (!this.rootDir && this.origin === _constants().COMPONENT_ORIGINS.NESTED) {
      throw new (_validationError().default)(`${errorMessage} rootDir attribute is missing`);
    }

    if (this.rootDir && !(0, _utils().isValidPath)(this.rootDir)) {
      throw new (_validationError().default)(`${errorMessage} rootDir attribute ${this.rootDir} is invalid`);
    }

    if (this.trackDir && this.origin !== _constants().COMPONENT_ORIGINS.AUTHORED) {
      throw new (_validationError().default)(`${errorMessage} trackDir attribute should be set for AUTHORED component only`);
    }

    if (this.originallySharedDir && this.origin === _constants().COMPONENT_ORIGINS.AUTHORED) {
      throw new (_validationError().default)(`${errorMessage} originallySharedDir attribute should be set for non AUTHORED components only`);
    }

    if (this.nextVersion && !this.nextVersion.version) {
      throw new (_validationError().default)(`${errorMessage} version attribute should be set when nextVersion prop is set`);
    }

    if (!this.files || !this.files.length) throw new (_validationError().default)(`${errorMessage} files list is missing`);
    this.files.forEach(file => {
      if (!(0, _utils().isValidPath)(file.relativePath)) {
        throw new (_validationError().default)(`${errorMessage} file path ${file.relativePath} is invalid`);
      }
    });
    const foundMainFile = this.files.find(file => file.relativePath === this.mainFile);

    if (!foundMainFile || _ramda().default.isEmpty(foundMainFile)) {
      throw new (_validationError().default)(`${errorMessage} mainFile ${this.mainFile} is not in the files list`);
    }

    const filesPaths = this.files.map(file => file.relativePath);
    const duplicateFiles = filesPaths.filter(file => filesPaths.filter(f => file.toLowerCase() === f.toLowerCase()).length > 1);

    if (duplicateFiles.length) {
      throw new (_validationError().default)(`${errorMessage} the following files are duplicated ${duplicateFiles.join(', ')}`);
    }

    if (this.trackDir) {
      const trackDir = this.trackDir;
      this.files.forEach(file => {
        if (!file.relativePath.startsWith(trackDir)) {
          throw new (_validationError().default)(`${errorMessage} a file path ${file.relativePath} is not in the trackDir ${trackDir}`);
        }
      });
    }
  }

}

exports.default = ComponentMap;