"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

require("core-js/modules/es.array.iterator");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FlattenedDependencyLoader = void 0;

function _bluebird() {
  const data = _interopRequireWildcard(require("bluebird"));

  _bluebird = function () {
    return data;
  };

  return data;
}

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _bitIds() {
  const data = _interopRequireDefault(require("../../bit-id/bit-ids"));

  _bitIds = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

function _componentDependencies() {
  const data = _interopRequireDefault(require("../../scope/component-dependencies"));

  _componentDependencies = function () {
    return data;
  };

  return data;
}

class FlattenedDependencyLoader {
  constructor(consumer, ignoreIds = new (_bitIds().default)(), loadComponentsFunc) {
    this.consumer = consumer;
    this.ignoreIds = ignoreIds;
    this.loadComponentsFunc = loadComponentsFunc;
    (0, _defineProperty2().default)(this, "cache", {});
  }

  load(component) {
    var _this = this;

    return (0, _bluebird().coroutine)(function* () {
      const dependencies = yield _this.loadManyDependencies(component.dependencies.getAllIds().difference(_this.ignoreIds));
      const devDependencies = yield _this.loadManyDependencies(component.devDependencies.getAllIds().difference(_this.ignoreIds));
      const extensionDependencies = yield _this.loadManyDependencies(component.extensions.extensionsBitIds.difference(_this.ignoreIds));

      const filterIgnoreIds = comps => {
        if (!_this.ignoreIds.length) {
          // workaround for old @teambit/cli. for some reason, comps sometimes have null/undefined
          // and as a result, even comps.filter(x => x) causes of errors later on.
          return comps;
        }

        return comps.filter(dep => !_this.ignoreIds.has(dep.id));
      };

      const filteredDeps = filterIgnoreIds(dependencies);
      const filteredDevDeps = filterIgnoreIds(devDependencies);
      const filteredExtDeps = filterIgnoreIds(extensionDependencies);
      yield _this.loadFlattenedRecursively(filteredDeps);
      yield _this.loadFlattenedRecursively(filteredDevDeps);
      yield _this.loadFlattenedRecursively(filteredExtDeps);
      return new (_componentDependencies().default)({
        component,
        dependencies: filteredDeps,
        devDependencies: filteredDevDeps,
        extensionDependencies: filteredExtDeps
      });
    })();
  }

  loadManyDependencies(dependenciesIds) {
    var _this2 = this;

    return (0, _bluebird().coroutine)(function* () {
      return _bluebird().default.mapSeries(dependenciesIds, dep => _this2.loadDependency(dep));
    })();
  }

  loadDependency(dependencyId) {
    var _this3 = this;

    return (0, _bluebird().coroutine)(function* () {
      if (!_this3.cache[dependencyId.toString()]) {
        const componentMap = _this3.consumer.bitMap.getComponentIfExist(dependencyId);

        const couldBeModified = componentMap && componentMap.origin !== _constants().COMPONENT_ORIGINS.NESTED;

        if (couldBeModified) {
          if (_this3.loadComponentsFunc) {
            const dependency = yield _this3.loadComponentsFunc([dependencyId]);
            if (!dependency.length || !dependency[0]) throw new Error(`unable to load ${dependencyId.toString()} using custom load function`);
            _this3.cache[dependencyId.toString()] = dependency[0];
          } else {
            _this3.cache[dependencyId.toString()] = yield _this3.consumer.loadComponentForCapsule(dependencyId);
          }

          return _this3.cache[dependencyId.toString()];
        } // for capsule, a dependency might have been installed as a package in the workspace, and as
        // such doesn't have a componentMap, which result in not stripping the sharedDir.
        // using the loadComponentWithDependenciesFromModel, all dependencies are loaded and their
        // shared dir is stripped. (see e2e-test of 'isolating with capsule' in dependencies-as-packages.e2e file)


        const componentWithDependenciesFromModel = yield _this3.consumer.loadComponentWithDependenciesFromModel(dependencyId, false);
        _this3.cache[dependencyId.toString()] = componentWithDependenciesFromModel.component.clone();
      }

      return _this3.cache[dependencyId.toString()];
    })();
  }

  loadFlattenedRecursively(deps, visited = []) {
    var _this4 = this;

    return (0, _bluebird().coroutine)(function* () {
      if (_ramda().default.isEmpty(deps)) return [];
      const notVisitedDeps = deps.filter(dep => !visited.includes(dep.id.toString()));
      const flattenedFromModel = yield _this4.loadFlattenedFromModel(notVisitedDeps);
      deps.push(...flattenedFromModel);
      const flattenedFromFs = yield _this4.loadFlattenedFromFs(notVisitedDeps);
      const newVisitedIds = deps.map(dep => dep.id.toString());
      deps.push(...flattenedFromFs);

      if (flattenedFromFs.length) {
        const newVisited = _ramda().default.uniq(visited.concat(newVisitedIds));

        yield _this4.loadFlattenedRecursively(deps, newVisited);
        return deps;
      }

      return deps;
    })();
  }

  loadFlattenedFromModel(deps) {
    var _this5 = this;

    return (0, _bluebird().coroutine)(function* () {
      const dependenciesFromModel = deps.filter(d => !d.loadedFromFileSystem);
      const flattenedIdsFromModel = dependenciesFromModel.map(d => d.flattenedDependencies);
      const flattenedFromModel = yield _this5.loadManyDependencies(_ramda().default.flatten(flattenedIdsFromModel));
      return flattenedFromModel;
    })();
  } // @todo: in case of out-of-sync, when a component has versions in the objects but the .bitmap
  // has the component without any version, this function result in "Maximum call stack size
  // exceeded" error.


  loadFlattenedFromFs(components) {
    var _this6 = this;

    return (0, _bluebird().coroutine)(function* () {
      const currentIds = _bitIds().default.fromArray(components.map(c => c.id));

      const ids = _ramda().default.flatten(components.filter(c => c.loadedFromFileSystem).map(c => c.getAllDependenciesIds()));

      const idsUniq = _bitIds().default.uniqFromArray(ids);

      const newIds = idsUniq.filter(id => !currentIds.has(id));
      if (_ramda().default.isEmpty(newIds)) return [];
      const deps = yield _this6.loadManyDependencies(newIds);
      if (_ramda().default.isEmpty(deps)) return [];
      return deps;
    })();
  }

}

exports.FlattenedDependencyLoader = FlattenedDependencyLoader;