import { BitId, BitIds } from '../../bit-id';
import { DivergeData } from '../../scope/component-ops/diverge-data';
import { Lane } from '../../scope/models';
import ModelComponent from '../../scope/models/model-component';
import Version from '../../scope/models/version';
import Scope from '../../scope/scope';
import BitMap from '../bit-map/bit-map';
import { ComponentOrigin } from '../bit-map/component-map';
import Component from '../component';
import { InvalidComponent } from '../component/consumer-component';
import Consumer from '../consumer';
export declare type ObjectsList = Promise<{
    [componentId: string]: Version;
}>;
export declare type ListScopeResult = {
    id: BitId;
    currentlyUsedVersion?: string | null | undefined;
    remoteVersion?: string;
    deprecated?: boolean;
};
export declare type DivergedComponent = {
    id: BitId;
    diverge: DivergeData;
};
export default class ComponentsList {
    consumer: Consumer;
    scope: Scope;
    bitMap: BitMap;
    _fromFileSystem: {
        [cacheKey: string]: Component[];
    };
    _fromObjectsIds: BitId[];
    _modelComponents: ModelComponent[];
    _invalidComponents: InvalidComponent[];
    _modifiedComponents: Component[];
    private _mergePendingComponents;
    constructor(consumer: Consumer);
    getModelComponents(): Promise<ModelComponent[]>;
    getFromObjects(): Promise<BitId[]>;
    getAuthoredAndImportedFromFS(): Promise<Component[]>;
    listModifiedComponents(load?: boolean): Promise<Array<BitId | Component>>;
    listOutdatedComponents(): Promise<Component[]>;
    listMergePendingComponents(): Promise<DivergedComponent[]>;
    listComponentsWithUnresolvedConflicts(): BitIds;
    listDuringMergeStateComponents(): BitIds;
    listSoftTaggedComponents(): BitId[];
    newModifiedAndAutoTaggedComponents(): Promise<Component[]>;
    authoredAndImportedComponents(): Promise<Component[]>;
    idsFromObjects(): Promise<BitIds>;
    listNewComponents(load?: boolean): Promise<BitIds | Component[]>;
    listTagPendingOfAllScope(includeImported?: boolean): Promise<BitId[]>;
    listTagPendingComponents(): Promise<BitIds>;
    listExportPendingComponentsIds(lane?: Lane | null): Promise<BitIds>;
    listNonNewComponentsIds(): Promise<BitIds>;
    updateIdsFromModelIfTheyOutOfSync(ids: BitIds): Promise<BitIds>;
    listExportPendingComponents(laneObj: Lane | null): Promise<ModelComponent[]>;
    listAutoTagPendingComponents(): Promise<Component[]>;
    idsFromBitMap(origin?: ComponentOrigin): BitId[];
    listAllIdsFromWorkspaceAndScope(): Promise<BitIds>;
    getFromFileSystem(origin?: ComponentOrigin): Promise<Component[]>;
    listInvalidComponents(): Promise<InvalidComponent[]>;
    listComponentsWithIndividualFiles(): Promise<Component[]>;
    listComponentsWithTrackDir(): Promise<Component[]>;
    getFromBitMap(origin?: ComponentOrigin): BitIds;
    getPathsForAllFilesOfAllComponents(origin?: ComponentOrigin, absolute?: boolean): string[];
    listScope(showRemoteVersion: boolean, includeNested: boolean, namespacesUsingWildcards?: string): Promise<ListScopeResult[]>;
    static listLocalScope(scope: Scope, namespacesUsingWildcards?: string): Promise<ListScopeResult[]>;
    static sortComponentsByName<T>(components: T): T;
    static filterComponentsByWildcard<T>(components: T, idsWithWildcard: string[] | string): T;
    static getUniqueComponents(components: Component[]): Component[];
    listComponentsByIdsWithWildcard(idsWithWildcard: string[]): BitId[];
}
