"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createNewLane;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _bitId() {
  const data = require("../../bit-id");

  _bitId = function () {
    return data;
  };

  return data;
}

function _generalError() {
  const data = _interopRequireDefault(require("../../error/general-error"));

  _generalError = function () {
    return data;
  };

  return data;
}

function _lane() {
  const data = _interopRequireDefault(require("../../scope/models/lane"));

  _lane = function () {
    return data;
  };

  return data;
}

function _utils() {
  const data = require("../../utils");

  _utils = function () {
    return data;
  };

  return data;
}

function _workspaceLane() {
  const data = _interopRequireDefault(require("../bit-map/workspace-lane"));

  _workspaceLane = function () {
    return data;
  };

  return data;
}

function createNewLane(_x, _x2, _x3) {
  return _createNewLane.apply(this, arguments);
}

function _createNewLane() {
  _createNewLane = (0, _bluebird().coroutine)(function* (consumer, laneName, laneComponents) {
    const lanes = yield consumer.scope.listLanes();

    if (lanes.find(lane => lane.name === laneName)) {
      throw new (_generalError().default)(`lane "${laneName}" already exists, to switch to this lane, please use "bit switch" command`);
    }

    if (!(0, _utils().isValidIdChunk)(laneName, false)) {
      // @todo: should we allow slash?
      throw new (_generalError().default)(`lane "${laneName}" has invalid characters. lane name can only contain alphanumeric, lowercase characters, and the following ["-", "_", "$", "!"]`);
    }

    const getDataToPopulateLaneObjectIfNeeded = /*#__PURE__*/function () {
      var _ref = (0, _bluebird().coroutine)(function* () {
        if (laneComponents) return laneComponents; // when branching from one lane to another, copy components from the origin lane
        // when branching from master, no need to copy anything

        const currentLaneObject = yield consumer.getCurrentLaneObject();
        return currentLaneObject ? currentLaneObject.components : [];
      });

      return function getDataToPopulateLaneObjectIfNeeded() {
        return _ref.apply(this, arguments);
      };
    }();

    const getDataToPopulateWorkspaceLaneIfNeeded = () => {
      if (laneComponents) return new (_bitId().BitIds)(); // if laneComponent, this got created when importing a remote lane
      // when branching from one lane to another, copy components from the origin workspace-lane
      // when branching from master, no need to copy anything

      const currentWorkspaceLane = consumer.bitMap.workspaceLane;
      return currentWorkspaceLane ? currentWorkspaceLane.ids : new (_bitId().BitIds)();
    };

    const newLane = _lane().default.create(laneName);

    const dataToPopulate = yield getDataToPopulateLaneObjectIfNeeded();
    newLane.setLaneComponents(dataToPopulate);
    yield consumer.scope.lanes.saveLane(newLane, true);

    const workspaceConfig = _workspaceLane().default.load(laneName, consumer.scope.getPath());

    workspaceConfig.ids = getDataToPopulateWorkspaceLaneIfNeeded();
    yield workspaceConfig.write();
    return newLane;
  });
  return _createNewLane.apply(this, arguments);
}