"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.HarmonyMigrator = void 0;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _chalk() {
  const data = _interopRequireDefault(require("chalk"));

  _chalk = function () {
    return data;
  };

  return data;
}

function _fsExtra() {
  const data = _interopRequireDefault(require("fs-extra"));

  _fsExtra = function () {
    return data;
  };

  return data;
}

function _path() {
  const data = _interopRequireDefault(require("path"));

  _path = function () {
    return data;
  };

  return data;
}

function _() {
  const data = require("..");

  _ = function () {
    return data;
  };

  return data;
}

function _featureToggle() {
  const data = require("../../api/consumer/lib/feature-toggle");

  _featureToggle = function () {
    return data;
  };

  return data;
}

function _statusCmd() {
  const data = require("../../cli/commands/public-cmds/status-cmd");

  _statusCmd = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

function _generalError() {
  const data = _interopRequireDefault(require("../../error/general-error"));

  _generalError = function () {
    return data;
  };

  return data;
}

function _logger() {
  const data = _interopRequireDefault(require("../../logger/logger"));

  _logger = function () {
    return data;
  };

  return data;
}

function _packageJsonFile() {
  const data = _interopRequireDefault(require("../component/package-json-file"));

  _packageJsonFile = function () {
    return data;
  };

  return data;
}

class HarmonyMigrator {
  constructor(consumer) {
    this.consumer = consumer;
    (0, _defineProperty2().default)(this, "messages", []);
  }

  migrate() {
    var _this = this;

    return (0, _bluebird().coroutine)(function* () {
      yield _this.initAsHarmony();
      const status = {
        individualFiles: [],
        changedToRootDir: []
      };

      const authorComponents = _this.consumer.bitMap.getAllComponents(_constants().COMPONENT_ORIGINS.AUTHORED);

      authorComponents.forEach(componentMap => {
        if (componentMap.rootDir) return;

        if (!componentMap.trackDir) {
          status.individualFiles.push(componentMap.id.toStringWithoutVersion());
          return;
        }

        componentMap.changeRootDirAndUpdateFilesAccordingly(componentMap.trackDir);
        status.changedToRootDir.push(componentMap.id.toStringWithoutVersion());

        _this.consumer.bitMap.markAsChanged();
      });

      _this.printResults(status);

      _this.printMessages();
    })();
  }

  initAsHarmony() {
    var _this2 = this;

    return (0, _bluebird().coroutine)(function* () {
      if (!_this2.consumer.isLegacy) return; // it's already Harmony.

      _this2.backupAndRemoveBitJson();

      _this2.backupAndRemoveBitPropInPkgJson();

      (0, _featureToggle().addFeature)(_featureToggle().HARMONY_FEATURE);

      const workspacePath = _this2.consumer.getPath(); // because Harmony feature is added, the load writes the workspace.jsonc because the configuration
      // files are now missing.


      const consumer = yield _().Consumer.load(workspacePath);

      if (!_fsExtra().default.existsSync(_path().default.join(workspacePath, _constants().WORKSPACE_JSONC))) {
        throw new Error('failed initializing the workspace as Harmony');
      }

      _this2.messages.push('congratulations! your workspace has been initialized as Harmony');

      _this2.consumer = consumer;
    })();
  }

  backupAndRemoveBitPropInPkgJson() {
    var _this3 = this;

    return (0, _bluebird().coroutine)(function* () {
      const packageJsonFile = yield _packageJsonFile().default.load(_this3.consumer.getPath());
      if (!packageJsonFile.packageJsonObject.bit) return;
      packageJsonFile.packageJsonObject['bit-legacy'] = packageJsonFile.packageJsonObject.bit;
      delete packageJsonFile.packageJsonObject.bit;
      yield packageJsonFile.write();

      _this3.messages.push(`Harmony doesn't work with the previous "bit" property in the package.json.
this property has been renamed to "bit-legacy". delete it once you don't need it.`);
    })();
  }

  backupAndRemoveBitJson() {
    const bitJsonPath = _path().default.join(this.consumer.getPath(), 'bit.json');

    if (!_fsExtra().default.existsSync(bitJsonPath)) return;

    _fsExtra().default.moveSync(bitJsonPath, `${bitJsonPath}.legacy`);

    this.messages.push(`Harmony doesn't work with your previous configuration file ${bitJsonPath}.
this file has been renamed to include ".legacy" suffix. delete it once you don't need it.`);
  }

  throwOnLegacy() {
    if (this.consumer.isLegacy) {
      throw new (_generalError().default)(`your workspace is working in legacy mode.
before starting the migration, please re-init the workspace as harmony by following these steps.
1. backup and remove your workspace settings (either bit.json or "bit" prop in package.json).
2. run "BIT_FEATURES=harmony bit init" (on Windows run "set BIT_FEATURES=harmony && bit init")`);
    }
  }

  printResults(results) {
    if (results.individualFiles.length) {
      _logger().default.console(_chalk().default.red(_statusCmd().individualFilesDesc));

      _logger().default.console(results.individualFiles.join('\n'));
    }

    if (results.changedToRootDir.length) {
      _logger().default.console(_chalk().default.green('the following components were successfully converted from trackDir to rootDir'));

      _logger().default.console(results.changedToRootDir.join('\n'));
    }

    this.messages.push('please run "bit status" to make sure the workspace is error-free before continue working');
  }

  printMessages() {
    this.messages.forEach(message => {
      _logger().default.console(_chalk().default.bold(`\n[-] ${message}`));
    });
  }

}

exports.HarmonyMigrator = HarmonyMigrator;