"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _logger() {
  const data = _interopRequireDefault(require("../logger/logger"));

  _logger = function () {
    return data;
  };

  return data;
}

function _network() {
  const data = require("../scope/network");

  _network = function () {
    return data;
  };

  return data;
}

function _ssh() {
  const data = require("../scope/network/ssh/ssh");

  _ssh = function () {
    return data;
  };

  return data;
}

function _utils() {
  const data = require("../utils");

  _utils = function () {
    return data;
  };

  return data;
}

function _exceptions() {
  const data = require("./exceptions");

  _exceptions = function () {
    return data;
  };

  return data;
}

/**
 * @ctx bit, primary, remote
 */
function isPrimary(alias) {
  return alias.includes('!');
}

class Remote {
  constructor(host, name, primary = false) {
    (0, _defineProperty2().default)(this, "primary", false);
    (0, _defineProperty2().default)(this, "host", void 0);
    (0, _defineProperty2().default)(this, "name", void 0);
    this.name = name || '';
    this.host = host;
    this.primary = primary;
  }

  connect(strategiesNames) {
    return (0, _network().connect)(this.host, strategiesNames);
  }

  toPlainObject() {
    return {
      host: this.host,
      name: this.name
    };
  }

  scope() {
    return this.connect().then(network => {
      return network.describeScope();
    });
  }

  list(namespacesUsingWildcards, strategiesNames = _ssh().DEFAULT_READ_STRATEGIES) {
    return this.connect(strategiesNames).then(network => network.list(namespacesUsingWildcards));
  }

  show(bitId, strategiesNames = _ssh().DEFAULT_READ_STRATEGIES) {
    return this.connect(strategiesNames).then(network => network.show(bitId));
  }

  graph(bitId, strategiesNames = _ssh().DEFAULT_READ_STRATEGIES) {
    // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
    return this.connect(strategiesNames).then(network => network.graph(bitId));
  }

  fetch(ids, fetchOptions, context, strategiesNames = _ssh().DEFAULT_READ_STRATEGIES) {
    return this.connect(strategiesNames).then(network => network.fetch(ids, fetchOptions, context));
  }

  latestVersions(bitIds, strategiesNames = _ssh().DEFAULT_READ_STRATEGIES) {
    // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
    return this.connect(strategiesNames).then(network => network.latestVersions(bitIds));
  }

  validate() {
    if (!(0, _utils().isBitUrl)(this.host)) throw new (_exceptions().InvalidRemote)();
  }

  push(componentObjects) {
    // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
    return (0, _network().connect)(this.host).then(network => network.push(componentObjects));
  }

  pushMany(objectList, context) {
    var _this = this;

    return (0, _bluebird().coroutine)(function* () {
      const network = yield (0, _network().connect)(_this.host);

      _logger().default.debug(`[-] Running pushMany on a remote`);

      const results = yield network.pushMany(objectList, context);

      _logger().default.debug('[-] Returning from a remote');

      return results;
    })();
  }

  deleteMany(ids, force, context, idsAreLanes = false) {
    // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
    return (0, _network().connect)(this.host).then(network => network.deleteMany(ids, force, context, idsAreLanes));
  }

  deprecateMany(ids, context) {
    return (0, _network().connect)(this.host).then(network => network.deprecateMany(ids, context));
  }

  undeprecateMany(ids, context) {
    return (0, _network().connect)(this.host).then(network => network.undeprecateMany(ids, context));
  }

  log(id) {
    return (0, _network().connect)(this.host).then(network => network.log(id));
  }

  listLanes(name, mergeData) {
    return (0, _network().connect)(this.host).then(network => network.listLanes(name, mergeData));
  }

  static load(name, host) {
    const primary = isPrimary(name);
    if (primary) name = (0, _utils().cleanBang)(name);
    return new Remote(name, host, primary);
  }

}

exports.default = Remote;