"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.promise");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = require("ramda");

  _ramda = function () {
    return data;
  };

  return data;
}

function _bitId() {
  const data = require("../bit-id");

  _bitId = function () {
    return data;
  };

  return data;
}

function _globalRemotes() {
  const data = _interopRequireDefault(require("../global-config/global-remotes"));

  _globalRemotes = function () {
    return data;
  };

  return data;
}

function _logger() {
  const data = _interopRequireDefault(require("../logger/logger"));

  _logger = function () {
    return data;
  };

  return data;
}

function _objectList() {
  const data = require("../scope/objects/object-list");

  _objectList = function () {
    return data;
  };

  return data;
}

function _utils() {
  const data = require("../utils");

  _utils = function () {
    return data;
  };

  return data;
}

function _exceptions() {
  const data = require("./exceptions");

  _exceptions = function () {
    return data;
  };

  return data;
}

function _remote() {
  const data = _interopRequireDefault(require("./remote"));

  _remote = function () {
    return data;
  };

  return data;
}

function _remoteResolver() {
  const data = _interopRequireDefault(require("./remote-resolver/remote-resolver"));

  _remoteResolver = function () {
    return data;
  };

  return data;
}

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2().default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

class Remotes extends Map {
  constructor(remotes = []) {
    super(remotes);
  }

  validate() {
    // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
    const primary = this.values.filter(remote => remote.primary);
    if (primary.length > 1) throw new (_exceptions().PrimaryOverloaded)();
    return this.forEach(remote => remote.validate());
  }

  resolve(scopeName, thisScope) {
    const remote = super.get(scopeName);
    if (remote) return Promise.resolve(remote); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    return (0, _remoteResolver().default)(scopeName, thisScope).then(scopeHost => {
      return new (_remote().default)(scopeHost, scopeName);
    });
  }

  isHub(scope) {
    // if a scope is listed as a remote, it doesn't go to the hub
    return !this.get(scope);
  }

  fetch(idsGroupedByScope, // option.type determines the id: component-id/lane-id/object-id (hash)
  thisScope, options = {}, context) {
    var _this = this;

    return (0, _bluebird().coroutine)(function* () {
      const fetchOptions = _objectSpread({
        type: 'component',
        withoutDependencies: false,
        includeArtifacts: false
      }, options);

      _logger().default.debug('[-] Running fetch on a remote, with the following options', fetchOptions);

      const objectLists = yield Promise.all(Object.keys(idsGroupedByScope).map( /*#__PURE__*/function () {
        var _ref = (0, _bluebird().coroutine)(function* (scopeName) {
          const remote = yield _this.resolve(scopeName, thisScope);
          return remote.fetch(idsGroupedByScope[scopeName], fetchOptions, context);
        });

        return function (_x) {
          return _ref.apply(this, arguments);
        };
      }()));

      _logger().default.debug('[-] Returning from a remote');

      return _objectList().ObjectList.mergeMultipleInstances(objectLists);
    })();
  }

  latestVersions(ids, thisScope) {
    var _this2 = this;

    return (0, _bluebird().coroutine)(function* () {
      const groupedIds = _this2._groupByScopeName(ids);

      const promises = Object.entries(groupedIds).map(([scopeName, scopeIds]) => _this2.resolve(scopeName, thisScope).then(remote => remote.latestVersions(scopeIds)));
      const components = yield Promise.all(promises);
      const flattenComponents = (0, _utils().flatten)(components);
      return flattenComponents.map(componentId => _bitId().BitId.parse(componentId, true));
    })();
  }
  /**
   * returns scope graphs of the given bit-ids.
   * it is possible to improve it by returning only the connected-graph of the given id and not the
   * entire scope graph. however, when asking for multiple ids in the same scope, which is more
   * likely to happen, it'll harm the performance.
   */


  scopeGraphs(ids, thisScope) {
    var _this3 = this;

    return (0, _bluebird().coroutine)(function* () {
      const groupedIds = _this3._groupByScopeName(ids);

      const graphsP = Object.keys(groupedIds).map( /*#__PURE__*/function () {
        var _ref2 = (0, _bluebird().coroutine)(function* (scopeName) {
          const remote = yield _this3.resolve(scopeName, thisScope);
          const dependencyGraph = yield remote.graph();
          dependencyGraph.setScopeName(scopeName);
          return dependencyGraph;
        });

        return function (_x2) {
          return _ref2.apply(this, arguments);
        };
      }());
      return Promise.all(graphsP);
    })();
  }

  _groupByScopeName(ids) {
    const byScope = (0, _ramda().groupBy)((0, _ramda().prop)('scope'));
    return byScope(ids);
  }

  toPlainObject() {
    const object = {};
    this.forEach(remote => {
      let name = remote.name;
      if (remote.primary) name = (0, _utils().prependBang)(remote.name);
      object[name] = remote.host;
    });
    return object;
  }

  static getScopeRemote(scopeName) {
    return Remotes.getGlobalRemotes().then(remotes => remotes.resolve(scopeName));
  }

  static getGlobalRemotes() {
    return _globalRemotes().default.load().then(globalRemotes => globalRemotes.toPlainObject()).then(remotes => Remotes.load(remotes));
  } // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
  // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!


  static load(remotes) {
    const models = [];
    if (!remotes) return new Remotes();
    (0, _utils().forEach)(remotes, (name, host) => {
      const remote = _remote().default.load(name, host); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
      // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!


      models.push([remote.name, remote]);
    });
    return new Remotes(models);
  }

}

exports.default = Remotes;