"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.promise");

require("core-js/modules/es.string.replace");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.exportManyBareScope = exportManyBareScope;
exports.exportMany = exportMany;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _graphlib() {
  const data = _interopRequireDefault(require("graphlib"));

  _graphlib = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _bitId() {
  const data = require("../../bit-id");

  _bitId = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

function _generalError() {
  const data = _interopRequireDefault(require("../../error/general-error"));

  _generalError = function () {
    return data;
  };

  return data;
}

function _enrichContextFromGlobal() {
  const data = _interopRequireDefault(require("../../hooks/utils/enrich-context-from-global"));

  _enrichContextFromGlobal = function () {
    return data;
  };

  return data;
}

function _laneId() {
  const data = require("../../lane-id/lane-id");

  _laneId = function () {
    return data;
  };

  return data;
}

function _logger() {
  const data = _interopRequireDefault(require("../../logger/logger"));

  _logger = function () {
    return data;
  };

  return data;
}

function _componentIdToPackageName() {
  const data = _interopRequireDefault(require("../../utils/bit/component-id-to-package-name"));

  _componentIdToPackageName = function () {
    return data;
  };

  return data;
}

function _replacePackageName() {
  const data = _interopRequireDefault(require("../../utils/string/replace-package-name"));

  _replacePackageName = function () {
    return data;
  };

  return data;
}

function _exceptions() {
  const data = require("../exceptions");

  _exceptions = function () {
    return data;
  };

  return data;
}

function _componentNeedsUpdate() {
  const data = _interopRequireDefault(require("../exceptions/component-needs-update"));

  _componentNeedsUpdate = function () {
    return data;
  };

  return data;
}

function _componentsGraph() {
  const data = require("../graph/components-graph");

  _componentsGraph = function () {
    return data;
  };

  return data;
}

function _models() {
  const data = require("../models");

  _models = function () {
    return data;
  };

  return data;
}

function _source() {
  const data = _interopRequireDefault(require("../models/source"));

  _source = function () {
    return data;
  };

  return data;
}

function _objects() {
  const data = require("../objects");

  _objects = function () {
    return data;
  };

  return data;
}

function _scopeRemotes() {
  const data = require("../scope-remotes");

  _scopeRemotes = function () {
    return data;
  };

  return data;
}

function _scopeComponentsImporter() {
  const data = _interopRequireDefault(require("./scope-components-importer"));

  _scopeComponentsImporter = function () {
    return data;
  };

  return data;
}

function _objectList() {
  const data = require("../objects/object-list");

  _objectList = function () {
    return data;
  };

  return data;
}

/**
 * @TODO there is no real difference between bare scope and a working directory scope - let's adjust terminology to avoid confusions in the future
 * saves a component into the objects directory of the remote scope, then, resolves its
 * dependencies, saves them as well. Finally runs the build process if needed on an isolated
 * environment.
 */
function exportManyBareScope(_x, _x2) {
  return _exportManyBareScope.apply(this, arguments);
}

function _exportManyBareScope() {
  _exportManyBareScope = (0, _bluebird().coroutine)(function* (scope, objectList) {
    _logger().default.debugAndAddBreadCrumb('exportManyBareScope', `started with ${objectList.objects.length} objects`);

    const mergedIds = yield mergeObjects(scope, objectList);

    _logger().default.debugAndAddBreadCrumb('exportManyBareScope', 'will try to importMany in case there are missing dependencies');

    const scopeComponentsImporter = _scopeComponentsImporter().default.getInstance(scope);

    yield scopeComponentsImporter.importMany(mergedIds, true, false); // resolve dependencies

    _logger().default.debugAndAddBreadCrumb('exportManyBareScope', 'successfully ran importMany');

    yield scope.objects.persist();

    _logger().default.debugAndAddBreadCrumb('exportManyBareScope', 'objects were written successfully to the filesystem');

    return mergedIds;
  });
  return _exportManyBareScope.apply(this, arguments);
}

function exportMany(_x3) {
  return _exportMany.apply(this, arguments);
}
/**
 * merge components into the scope.
 *
 * a component might have multiple versions that some where merged and some were not.
 * the BitIds returned here includes the versions that were merged. so it could contain multiple
 * ids of the same component with different versions
 */


function _exportMany() {
  _exportMany = (0, _bluebird().coroutine)(function* ({
    scope,
    ids,
    remoteName,
    context = {},
    includeDependencies = false,
    // kind of fork. by default dependencies only cached, with this, their scope-name is changed
    changeLocallyAlthoughRemoteIsDifferent = false,
    // by default only if remote stays the same the component is changed from staged to exported
    codemod = false,
    lanesObjects = [],
    allVersions,
    idsWithFutureScope
  }) {
    _logger().default.debugAndAddBreadCrumb('scope.exportMany', 'ids: {ids}', {
      ids: ids.toString()
    });

    (0, _enrichContextFromGlobal().default)(context);

    if (lanesObjects.length && !remoteName) {
      throw new Error('todo: implement export lanes to default scopes after tracking lanes local:remote is implemented');
    }

    if (includeDependencies) {
      const dependenciesIds = yield getDependenciesImportIfNeeded();
      ids.push(...dependenciesIds);
      ids = _bitId().BitIds.uniqFromArray(ids);
    }

    const remotes = yield (0, _scopeRemotes().getScopeRemotes)(scope);

    if (remoteName) {
      _logger().default.debugAndAddBreadCrumb('export-scope-components', 'export all ids to one remote');

      return exportIntoRemote(remoteName, ids, lanesObjects);
    }

    _logger().default.debugAndAddBreadCrumb('export-scope-components', 'export ids to multiple remotes');

    const groupedByScope = yield sortAndGroupByScope();
    const groupedByScopeString = groupedByScope.map(item => `scope "${item.scopeName}": ${item.ids.toString()}`).join(', ');

    _logger().default.debug(`export-scope-components, export to the following scopes ${groupedByScopeString}`);

    const results = yield (0, _bluebird().mapSeries)(groupedByScope, result => exportIntoRemote(result.scopeName, result.ids));
    return {
      newIdsOnRemote: _ramda().default.flatten(results.map(r => r.newIdsOnRemote)),
      exported: _bitId().BitIds.uniqFromArray(_ramda().default.flatten(results.map(r => r.exported))),
      updatedLocally: _bitId().BitIds.uniqFromArray(_ramda().default.flatten(results.map(r => r.updatedLocally)))
    };

    function exportIntoRemote(_x12, _x13) {
      return _exportIntoRemote.apply(this, arguments);
    }
    /**
     * the topological sort is needed in case components have dependencies in other scopes.
     * without sorting, in case remoteA/compA depends on remoteB/compB and remoteA/compA was exported
     * first, remoteA will throw an error that remoteB/compB was not found.
     * sorting the components topologically, ensure we export remoteB/compB first.
     *
     * there are a few cases to consider:
     * 1) in case there are cycle dependencies between the scopes, it's impossible to toposort.
     * 2) the cycle dependencies can be between different versions. e.g. remoteA/compA@0.0.1 requires
     * remoteB/compB@0.0.1 and remoteB/compB@0.0.2 requires remoteA/compA@0.0.2.
     * that's why when building the graph we take all versions into account and build the graph
     * without the version number, so then we could let the graph's algorithm finding the cycle.
     * 3) it's possible to have circle dependencies inside the same scope, and non-circle
     * dependencies between the different scopes. in this case, the toposort should be done after
     * removing the ids participated in the circular.
     *
     * once the sort is done, it returns an array of { scopeName: string; ids: BitIds }.
     * keep in mind that this array might have multiple items with the same scopeName, that's totally
     * valid and it will cause multiple round-trip to the same scope. there is no other way around
     * it.
     * the sort is done after eliminating circles, so it's possible to execute topsort. once the
     * components are topological sorted, they are added one by one to the results array. If the last
     * item in the array has the same scope as the currently inserted component, it can be added to
     * the same scope group. otherwise, a new item needs to be added to the array with the new scope.
     */


    function _exportIntoRemote() {
      _exportIntoRemote = (0, _bluebird().coroutine)(function* (remoteNameStr, bitIds, lanes = []) {
        bitIds.throwForDuplicationIgnoreVersion();
        const remote = yield remotes.resolve(remoteNameStr, scope);
        const componentObjects = yield (0, _bluebird().mapSeries)(bitIds, id => scope.sources.getObjects(id));

        const idsToChangeLocally = _bitId().BitIds.fromArray(bitIds.filter(id => !id.scope || id.scope === remoteNameStr || changeLocallyAlthoughRemoteIsDifferent));

        const idsAndObjectsP = lanes.map(laneObj => laneObj.collectObjectsById(scope.objects));

        const idsAndObjects = _ramda().default.flatten(yield Promise.all(idsAndObjectsP));

        const componentsAndObjects = [];
        const objectList = new (_objectList().ObjectList)();

        const processComponentObjects = /*#__PURE__*/function () {
          var _ref = (0, _bluebird().coroutine)(function* (componentObject) {
            const componentAndObject = componentObject.toObjects();
            const localVersions = componentAndObject.component.getLocalVersions();
            idsAndObjects.forEach(idAndObjects => {
              if (componentAndObject.component.toBitId().isEqual(idAndObjects.id)) {
                // @todo: remove duplication. check whether the same hash already exist, and don't push it.
                componentAndObject.objects.push(...idAndObjects.objects);
              }
            });
            componentAndObject.component.clearStateData();
            const didConvertScope = yield convertToCorrectScope(scope, componentAndObject, remoteNameStr, includeDependencies, bitIds, codemod);
            const remoteObj = {
              url: remote.host,
              name: remote.name,
              date: Date.now().toString()
            };
            componentAndObject.component.addScopeListItem(remoteObj);

            if (idsToChangeLocally.hasWithoutScopeAndVersion(componentAndObject.component.toBitId())) {
              // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
              componentsAndObjects.push(componentAndObject);
            } else {
              // the component should not be changed locally. only add the new scope to the scope-list
              const componentAndObjectCloned = componentObject.toObjects();
              componentAndObjectCloned.component.addScopeListItem(remoteObj); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

              componentsAndObjects.push(componentAndObjectCloned);
            }

            const componentBuffer = yield componentAndObject.component.compress();
            const componentData = {
              ref: componentAndObject.component.hash(),
              buffer: componentBuffer
            };

            const getObjectsData = /*#__PURE__*/function () {
              var _ref2 = (0, _bluebird().coroutine)(function* () {
                // @todo currently, for lanes (componentAndObject.component.head) this optimization is skipped.
                // it should be enabled with a different mechanism
                if (allVersions || includeDependencies || didConvertScope || componentAndObject.component.head || lanes.length) {
                  // only when really needed (e.g. fork or version changes), collect all versions objects
                  return Promise.all(componentAndObject.objects.map( /*#__PURE__*/function () {
                    var _ref3 = (0, _bluebird().coroutine)(function* (obj) {
                      return {
                        ref: obj.hash(),
                        buffer: yield obj.compress()
                      };
                    });

                    return function (_x15) {
                      return _ref3.apply(this, arguments);
                    };
                  }()));
                } // when possible prefer collecting only new/local versions. the server has already
                // the rest, so no point of sending them.


                return componentAndObject.component.collectVersionsObjects(scope.objects, localVersions);
              });

              return function getObjectsData() {
                return _ref2.apply(this, arguments);
              };
            }();

            const objectsBuffer = yield getObjectsData();
            objectList.addIfNotExist([componentData, ...objectsBuffer]);
          });

          return function processComponentObjects(_x14) {
            return _ref.apply(this, arguments);
          };
        }(); // don't use Promise.all, otherwise, it'll throw "JavaScript heap out of memory" on a large set of data


        yield (0, _bluebird().mapSeries)(componentObjects, processComponentObjects);
        const lanesData = yield Promise.all(lanes.map( /*#__PURE__*/function () {
          var _ref4 = (0, _bluebird().coroutine)(function* (lane) {
            lane.components.forEach(c => {
              c.id = c.id.changeScope(remoteName);
            });
            return {
              ref: lane.hash(),
              buffer: yield lane.compress()
            };
          });

          return function (_x16) {
            return _ref4.apply(this, arguments);
          };
        }()));
        objectList.addIfNotExist(lanesData);
        let exportedIds;

        try {
          exportedIds = yield remote.pushMany(objectList, context);

          _logger().default.debugAndAddBreadCrumb('exportMany', 'successfully pushed all ids to the bare-scope, going to save them back to local scope');
        } catch (err) {
          _logger().default.warnAndAddBreadCrumb('exportMany', 'failed pushing ids to the bare-scope');

          return Promise.reject(err);
        }

        yield Promise.all(idsToChangeLocally.map(id => scope.sources.removeComponentById(id))); // @ts-ignore

        idsToChangeLocally.forEach(id => scope.createSymlink(id, remoteNameStr));
        componentsAndObjects.forEach(componentObject => scope.sources.put(componentObject)); // update lanes

        yield Promise.all(lanes.map( /*#__PURE__*/function () {
          var _ref5 = (0, _bluebird().coroutine)(function* (lane) {
            if (idsToChangeLocally.length) {
              // otherwise, we don't want to update scope-name of components in the lane object
              scope.objects.add(lane); // this is needed so later on we can add the tracking data and update .bitmap
              // @todo: support having a different name on the remote by a flag

              lane.remoteLaneId = _laneId().RemoteLaneId.from(lane.name, remoteNameStr);
            }

            yield scope.objects.remoteLanes.syncWithLaneObject(remoteNameStr, lane);
          });

          return function (_x17) {
            return _ref5.apply(this, arguments);
          };
        }()));
        const currentLane = scope.lanes.getCurrentLaneName();

        if (currentLane === _constants().DEFAULT_LANE && !lanes.length) {
          // all exported from master
          const remoteLaneId = _laneId().RemoteLaneId.from(_constants().DEFAULT_LANE, remoteNameStr);

          yield scope.objects.remoteLanes.loadRemoteLane(remoteLaneId);
          componentsAndObjects.forEach(({
            component
          }) => {
            scope.objects.remoteLanes.addEntry(remoteLaneId, component.toBitId(), component.getHead());
          });
        }

        yield scope.objects.persist();
        const newIdsOnRemote = exportedIds.map(id => _bitId().BitId.parse(id, true)); // remove version. exported component might have multiple versions exported

        const idsWithRemoteScope = newIdsOnRemote.map(id => id.changeVersion(undefined));

        const idsWithRemoteScopeUniq = _bitId().BitIds.uniqFromArray(idsWithRemoteScope);

        return {
          newIdsOnRemote,
          exported: idsWithRemoteScopeUniq,
          updatedLocally: _bitId().BitIds.fromArray(idsWithRemoteScopeUniq.filter(id => idsToChangeLocally.hasWithoutScopeAndVersion(id)))
        };
      });
      return _exportIntoRemote.apply(this, arguments);
    }

    function sortAndGroupByScope() {
      return _sortAndGroupByScope.apply(this, arguments);
    }

    function _sortAndGroupByScope() {
      _sortAndGroupByScope = (0, _bluebird().coroutine)(function* () {
        const grouped = ids.toGroupByScopeName(idsWithFutureScope);
        const groupedArrayFormat = Object.keys(grouped).map(scopeName => ({
          scopeName,
          ids: grouped[scopeName]
        }));

        if (Object.keys(grouped).length <= 1) {
          return groupedArrayFormat;
        } // when exporting to multiple scopes, there is a chance of dependencies between the different scopes


        const componentsAndVersions = yield scope.getComponentsAndAllLocalUnexportedVersions(ids);
        const graph = (0, _componentsGraph().buildOneGraphForComponentsAndMultipleVersions)(componentsAndVersions);

        const cycles = _graphlib().default.alg.findCycles(graph);

        const groupedArraySorted = [];

        const addToGroupedSorted = id => {
          if (groupedArraySorted.length) {
            const lastItem = groupedArraySorted[groupedArraySorted.length - 1];

            if (lastItem.scopeName === id.scope) {
              lastItem.ids.push(id);
              return;
            }
          }

          const idWithFutureScope = idsWithFutureScope.searchWithoutScopeAndVersion(id);

          if (idWithFutureScope) {
            groupedArraySorted.push({
              scopeName: idWithFutureScope.scope,
              ids: new (_bitId().BitIds)(id)
            });
          } // otherwise, it's in the graph, but not in the idWithFutureScope array. this is probably just a
          // dependency of one of the pending-export ids, and that dependency is not supposed to be
          // export, so just ignore it.

        };

        if (cycles.length) {
          const cyclesWithMultipleScopes = cycles.filter(cycle => {
            const bitIds = cycle.map(s => graph.node(s));
            const firstScope = bitIds[0].scope;
            return bitIds.some(id => id.scope !== firstScope);
          });

          if (cyclesWithMultipleScopes.length) {
            throw new (_generalError().default)(`fatal: unable to export. the following components have circular dependencies between two or more scopes
${cyclesWithMultipleScopes.map(c => c.join(', ')).join('\n')}
please untag the problematic components and eliminate the circle between the scopes.
tip: use "bit graph [--all-versions]" to get a visual look of the circular dependencies`);
          } // there are circles but they are all from the same scope, add them to groupedArraySorted
          // first, then, remove from the graph, so it will be possible to execute topsort


          cycles.forEach(cycle => {
            cycle.forEach(node => {
              const id = graph.node(node);
              addToGroupedSorted(id);
              graph.removeNode(node);
            });
          });
        } // @todo: optimize in case each one of the ids has all its dependencies from the same scope,
        // return groupedArrayFormat


        let sortedComponents;

        try {
          sortedComponents = _graphlib().default.alg.topsort(graph);
        } catch (err) {
          // should never arrive here, it's just a precaution, as topsort doesn't fail nicely
          _logger().default.error(err);

          throw new Error(`fatal: graphlib was unable to topsort the components. circles: ${cycles}`);
        }

        const sortedComponentsIds = sortedComponents.map(s => graph.node(s)).reverse();
        sortedComponentsIds.forEach(id => addToGroupedSorted(id));
        return groupedArraySorted;
      });
      return _sortAndGroupByScope.apply(this, arguments);
    }

    function getDependenciesImportIfNeeded() {
      return _getDependenciesImportIfNeeded.apply(this, arguments);
    }

    function _getDependenciesImportIfNeeded() {
      _getDependenciesImportIfNeeded = (0, _bluebird().coroutine)(function* () {
        const scopeComponentImporter = new (_scopeComponentsImporter().default)(scope);
        const versionsDependencies = yield scopeComponentImporter.importManyWithAllVersions(ids, true, true);

        const allDependencies = _ramda().default.flatten(versionsDependencies.map(versionDependencies => versionDependencies.allDependencies));

        return allDependencies.map(componentVersion => componentVersion.component.toBitId());
      });
      return _getDependenciesImportIfNeeded.apply(this, arguments);
    }
  });
  return _exportMany.apply(this, arguments);
}

function mergeObjects(_x4, _x5) {
  return _mergeObjects.apply(this, arguments);
}
/**
 * Component and dependencies id changes:
 * When exporting components with dependencies to a bare-scope, some of the dependencies may be created locally and as
 * a result their scope-name is null. Once the bare-scope gets the components, it needs to convert these scope names
 * to the bare-scope name.
 * Since the changes it does affect the Version objects, the version REF of a component, needs to be changed as well.
 *
 * Dist code changes:
 * see https://github.com/teambit/bit/issues/1770 for complete info
 * some compilers require the links to be part of the bundle, change the component name in these
 * files from the id without scope to the id with the scope
 * e.g. `@bit/utils.is-string` becomes `@bit/scope-name.utils.is-string`.
 * these files changes need to be done regardless the "--rewire" flag.
 *
 * Source code changes (codemod):
 * when "--rewire" flag is used, import/require statement should be changed from the old scope-name
 * to the new scope-name. Or from no-scope to the new scope.
 */


function _mergeObjects() {
  _mergeObjects = (0, _bluebird().coroutine)(function* (scope, objectList) {
    const bitObjectList = yield objectList.toBitObjects();
    const components = bitObjectList.getComponents();
    const lanesObjects = bitObjectList.getLanes();
    const versions = bitObjectList.getVersions();

    _logger().default.debugAndAddBreadCrumb('export-scope-components.mergeObjects', `Going to merge ${components.length} components, ${lanesObjects.length} lanes`);

    const mergeResults = yield Promise.all(components.map( /*#__PURE__*/function () {
      var _ref6 = (0, _bluebird().coroutine)(function* (component) {
        try {
          const result = yield scope.sources.merge(component, versions, true, false);
          return result;
        } catch (err) {
          if (err instanceof _exceptions().MergeConflict || err instanceof _componentNeedsUpdate().default) {
            return err; // don't throw. instead, get all components with merge-conflicts
          }

          throw err;
        }
      });

      return function (_x18) {
        return _ref6.apply(this, arguments);
      };
    }())); // components and lanes can't be just added, they need to be carefully merged.

    const objectsToAdd = bitObjectList.getAllExceptComponentsAndLanes();
    scope.sources.putObjects(objectsToAdd);
    const mergeLaneResultsP = lanesObjects.map(laneObject => scope.sources.mergeLane(laneObject, false));

    const mergeLaneResults = _ramda().default.flatten(yield Promise.all(mergeLaneResultsP));

    const componentsWithConflicts = mergeResults.filter(result => result instanceof _exceptions().MergeConflict);
    const componentsNeedUpdate = [...mergeResults.filter(result => result instanceof _componentNeedsUpdate().default), ...mergeLaneResults.filter(result => result instanceof _componentNeedsUpdate().default)];

    if (componentsWithConflicts.length || componentsNeedUpdate.length) {
      // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
      const idsAndVersions = componentsWithConflicts.map(c => ({
        id: c.id,
        versions: c.versions
      }));

      const idsAndVersionsWithConflicts = _ramda().default.sortBy(_ramda().default.prop('id'), idsAndVersions); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!


      const idsOfNeedUpdateComps = _ramda().default.sortBy(_ramda().default.prop('id'), componentsNeedUpdate.map(c => ({
        id: c.id,
        lane: c.lane
      })));

      scope.objects.clearCache(); // just in case this error is caught. we don't want to persist anything by mistake.

      throw new (_exceptions().MergeConflictOnRemote)(idsAndVersionsWithConflicts, idsOfNeedUpdateComps);
    } // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!


    const mergedComponents = mergeResults.filter(({
      mergedVersions
    }) => mergedVersions.length);
    const mergedLanesComponents = mergeLaneResults.filter(({
      mergedVersions
    }) => mergedVersions.length);

    const getMergedIds = ({
      mergedComponent,
      mergedVersions
    }) => mergedVersions.map(version => mergedComponent.toBitId().changeVersion(version)); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!


    return _bitId().BitIds.fromArray(_ramda().default.flatten([...mergedComponents, ...mergedLanesComponents].map(getMergedIds)));
  });
  return _mergeObjects.apply(this, arguments);
}

function convertToCorrectScope(_x6, _x7, _x8, _x9, _x10, _x11) {
  return _convertToCorrectScope.apply(this, arguments);
}

function _convertToCorrectScope() {
  _convertToCorrectScope = (0, _bluebird().coroutine)(function* (scope, componentsObjects, remoteScope, fork, exportingIds, codemod) {
    // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
    const versionsObjects = componentsObjects.objects.filter(object => object instanceof _models().Version);
    const haveVersionsChanged = yield Promise.all(versionsObjects.map( /*#__PURE__*/function () {
      var _ref7 = (0, _bluebird().coroutine)(function* (objectVersion) {
        const hashBefore = objectVersion.hash().toString();
        const didCodeMod = yield _replaceSrcOfVersionIfNeeded(objectVersion);
        const didDependencyChange = changeDependencyScope(objectVersion);
        changeExtensionsScope(objectVersion); // @todo: after v15 is deployed, remove the following code until the next "// END" comment.
        // this is currently needed because remote-servers with older code still saving Version
        // objects into the calculated hash path and not into the originally created hash.
        // in this scenario, the calculated hash is different than the original hash due to the scope
        // changes. if we don't do this hash replacement, these remote servers will write the version
        // objects into different paths and then throw an error of component-not-found due to failure
        // finding the Version objects on the fs.

        const hashAfter = objectVersion.calculateHash().toString();
        const isTag = componentsObjects.component.getTagOfRefIfExists(objectVersion.hash());

        if (isTag && hashBefore !== hashAfter) {
          if (!didCodeMod && !didDependencyChange) {
            throw new Error('hash should not be changed if there was not any dependency scope changes nor codemod');
          }

          objectVersion._hash = hashAfter;

          _logger().default.debugAndAddBreadCrumb('scope._convertToCorrectScope', `switching {id} version hash from ${hashBefore} to ${hashAfter}`, {
            id: componentsObjects.component.id().toString()
          });

          const versions = componentsObjects.component.versions;
          Object.keys(versions).forEach(version => {
            if (versions[version].toString() === hashBefore) {
              versions[version] = _objects().Ref.from(hashAfter);
            }
          });

          if (componentsObjects.component.getHeadStr() === hashBefore) {
            componentsObjects.component.setHead(_objects().Ref.from(hashAfter));
          }

          versionsObjects.forEach(versionObj => {
            versionObj.parents = versionObj.parents.map(parent => {
              if (parent.toString() === hashBefore) return _objects().Ref.from(hashAfter);
              return parent;
            });
          });
        } // END DELETION OF BIT > v15.


        return didCodeMod || didDependencyChange;
      });

      return function (_x19) {
        return _ref7.apply(this, arguments);
      };
    }()));
    const hasComponentChanged = remoteScope !== componentsObjects.component.scope;
    componentsObjects.component.scope = remoteScope; // return true if one of the versions has changed or the component itself

    return haveVersionsChanged.some(x => x) || hasComponentChanged;

    function changeDependencyScope(version) {
      let hasChanged = false;
      version.getAllDependencies().forEach(dependency => {
        const updatedScope = getIdWithUpdatedScope(dependency.id);

        if (!updatedScope.isEqual(dependency.id)) {
          hasChanged = true;
          dependency.id = updatedScope;
        }
      });
      const flattenedFields = ['flattenedDependencies', 'flattenedDevDependencies'];
      flattenedFields.forEach(flattenedField => {
        const ids = version[flattenedField];
        const needsChange = ids.some(id => id.scope !== remoteScope);

        if (needsChange) {
          version[flattenedField] = getBitIdsWithUpdatedScope(ids);
          hasChanged = true;
        }
      });
      return hasChanged;
    }

    function changeExtensionsScope(version) {
      let hasChanged = false;
      version.extensions.forEach(ext => {
        if (ext.extensionId) {
          const updatedScope = getIdWithUpdatedScope(ext.extensionId);

          if (!updatedScope.isEqual(ext.extensionId)) {
            hasChanged = true;
            ext.extensionId = updatedScope;
          }
        }

        if (ext.name === _constants().Extensions.dependencyResolver && ext.data && ext.data.dependencies) {
          ext.data.dependencies.forEach(dep => {
            const id = new (_bitId().BitId)(dep.componentId);
            const oldScope = id.scope;
            const updatedScope = getIdWithUpdatedScope(id);

            if (!updatedScope.isEqual(id)) {
              hasChanged = true;
              dep.componentId = updatedScope;
            }

            if (oldScope && dep.packageName.includes(oldScope)) {
              dep.packageName = dep.packageName.replace(oldScope, updatedScope.scope);
            }
          });
        }
      });
      return hasChanged;
    }

    function getIdWithUpdatedScope(dependencyId) {
      if (dependencyId.scope === remoteScope) {
        return dependencyId; // nothing has changed
      }

      if (!dependencyId.scope || fork || exportingIds.hasWithoutVersion(dependencyId)) {
        const depId = _models().ModelComponent.fromBitId(dependencyId); // todo: use 'load' for async and switch the foreach with map.


        const dependencyObject = scope.objects.loadSync(depId.hash());

        if (dependencyObject instanceof _models().Symlink) {
          return dependencyId.changeScope(dependencyObject.realScope);
        }

        return dependencyId.changeScope(remoteScope);
      }

      return dependencyId;
    }

    function getBitIdsWithUpdatedScope(bitIds) {
      const updatedIds = bitIds.map(id => getIdWithUpdatedScope(id));
      return _bitId().BitIds.fromArray(updatedIds);
    }

    function _replaceSrcOfVersionIfNeeded(_x20) {
      return _replaceSrcOfVersionIfNeeded2.apply(this, arguments);
    }
    /**
     * in the following cases it is needed to change files content:
     * 1. fork. exporting components of scope-a to scope-b. requirement: 1) --rewire flag. 2) id must have scope.
     * 2. dists. changing no-scope to current scope. requirement: 1) id should not have scope.
     * 3. no-scope. changing src no-scope to current scope. requirement: 1) --rewire flag. 2) id should not have scope.
     *
     * according to these three. if --rewire was not used and id has scope, no need to do anything.
     *
     * in the following conditions the process should stop and ask for --rewire flag:
     * 1. --rewire flag was not entered.
     * 2. id does not have scope.
     * 3. the file is not a dist file.
     * 4. the file content has pkg name without scope-name.
     */


    function _replaceSrcOfVersionIfNeeded2() {
      _replaceSrcOfVersionIfNeeded2 = (0, _bluebird().coroutine)(function* (version) {
        let hasVersionChanged = false;

        const processFile = /*#__PURE__*/function () {
          var _ref8 = (0, _bluebird().coroutine)(function* (file, isDist) {
            const newFileObject = yield _createNewFileIfNeeded(version, file, isDist);

            if (newFileObject && (codemod || isDist)) {
              file.file = newFileObject.hash();
              componentsObjects.objects.push(newFileObject);
              hasVersionChanged = true;
            }

            return null;
          });

          return function processFile(_x24, _x25) {
            return _ref8.apply(this, arguments);
          };
        }();

        yield Promise.all(version.files.map(file => processFile(file, false)));
        yield Promise.all((version.dists || []).map(file => processFile(file, true)));
        return hasVersionChanged;
      });
      return _replaceSrcOfVersionIfNeeded2.apply(this, arguments);
    }

    function _createNewFileIfNeeded(_x21, _x22, _x23) {
      return _createNewFileIfNeeded2.apply(this, arguments);
    }

    function _createNewFileIfNeeded2() {
      _createNewFileIfNeeded2 = (0, _bluebird().coroutine)(function* (version, file, isDist) {
        // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
        const currentHash = file.file; // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

        const fileObject = yield scope.objects.load(currentHash);
        const fileString = fileObject.contents.toString();
        const dependenciesIds = version.getAllDependencies().map(d => d.id);
        const componentId = componentsObjects.component.toBitId();
        const allIds = [...dependenciesIds, componentId];
        let newFileString = fileString;
        allIds.forEach(id => {
          if (id.scope === remoteScope) {
            return; // nothing to do, the remote has not changed
          }

          const idWithNewScope = id.changeScope(remoteScope);
          const pkgNameWithOldScope = (0, _componentIdToPackageName().default)({
            id,
            bindingPrefix: componentsObjects.component.bindingPrefix,
            extensions: version.extensions
          });

          if (!codemod) {
            // use did not enter --rewire flag
            if (id.hasScope()) {
              return; // because only --rewire is permitted to change from scope-a to scope-b.
            } // dists can change no-scope to scope without --rewire flag. if this is not a dist file
            // and the file needs to change from no-scope to scope, it needs to --rewire flag.
            // for non-legacy the no-scope is not possible, so no need to check for it.


            if (!isDist && fileString.includes(pkgNameWithOldScope) && version.isLegacy) {
              throw new (_generalError().default)(`please use "--rewire" flag to fix the import/require statements "${pkgNameWithOldScope}" in "${file.relativePath}" file of ${componentId.toString()},
the current import/require module has no scope-name, which result in an invalid module path upon import`);
            }
          } // at this stage, we know that either 1) --rewire was used. 2) it's dist and id doesn't have scope-name
          // in both cases, if the file has the old package-name, it should be replaced to the new one.


          const pkgNameWithNewScope = (0, _componentIdToPackageName().default)({
            id: idWithNewScope,
            bindingPrefix: componentsObjects.component.bindingPrefix,
            extensions: version.extensions
          }); // replace old scope to a new scope (e.g. '@bit/old-scope.is-string' => '@bit/new-scope.is-string')
          // or no-scope to a new scope. (e.g. '@bit/is-string' => '@bit/new-scope.is-string')

          newFileString = (0, _replacePackageName().default)(newFileString, pkgNameWithOldScope, pkgNameWithNewScope);
        });

        if (newFileString !== fileString) {
          return _source().default.from(Buffer.from(newFileString));
        }

        return null;
      });
      return _createNewFileIfNeeded2.apply(this, arguments);
    }
  });
  return _convertToCorrectScope.apply(this, arguments);
}