"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.promise");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FlattenedDependenciesGetter = void 0;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _graphlib() {
  const data = _interopRequireDefault(require("graphlib"));

  _graphlib = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _bitId() {
  const data = require("../../bit-id");

  _bitId = function () {
    return data;
  };

  return data;
}

function _generalError() {
  const data = _interopRequireDefault(require("../../error/general-error"));

  _generalError = function () {
    return data;
  };

  return data;
}

function _exceptions() {
  const data = require("../exceptions");

  _exceptions = function () {
    return data;
  };

  return data;
}

function _flattenDependencies() {
  const data = require("../flatten-dependencies");

  _flattenDependencies = function () {
    return data;
  };

  return data;
}

function _componentsGraph() {
  const data = require("../graph/components-graph");

  _componentsGraph = function () {
    return data;
  };

  return data;
}

function _scopeComponentsImporter() {
  const data = _interopRequireDefault(require("./scope-components-importer"));

  _scopeComponentsImporter = function () {
    return data;
  };

  return data;
}

class FlattenedDependenciesGetter {
  constructor(scope, components) {
    this.scope = scope;
    this.components = components;
    (0, _defineProperty2().default)(this, "allDependenciesGraphs", void 0);
    (0, _defineProperty2().default)(this, "cache", {});
    (0, _defineProperty2().default)(this, "notFoundDependencies", new (_bitId().BitIds)());
  }

  populateFlattenedDependencies() {
    var _this = this;

    return (0, _bluebird().coroutine)(function* () {
      _this.allDependenciesGraphs = (0, _componentsGraph().buildComponentsGraph)(_this.components);
      yield _this.importExternalDependenciesInBulk();
      yield Promise.all(_this.components.map( /*#__PURE__*/function () {
        var _ref = (0, _bluebird().coroutine)(function* (component) {
          const {
            flattenedDependencies,
            flattenedDevDependencies
          } = yield _this.getAllFlattenedDependencies(component.id);
          component.flattenedDependencies = flattenedDependencies;
          component.flattenedDevDependencies = flattenedDevDependencies;
        });

        return function (_x) {
          return _ref.apply(this, arguments);
        };
      }()));
    })();
  }

  importExternalDependenciesInBulk() {
    var _this2 = this;

    return (0, _bluebird().coroutine)(function* () {
      const allGraphs = Object.values(_this2.allDependenciesGraphs);

      const allDependencies = _this2.components.map(component => {
        return allGraphs.map(graph => getEdges(graph, component.id.toString()));
      });

      const idsStr = _ramda().default.uniq(_ramda().default.flatten(allDependencies));

      const bitIds = idsStr.filter(id => id).map(idStr => {
        const graphWithId = allGraphs.find(g => g.node(idStr));
        return graphWithId === null || graphWithId === void 0 ? void 0 : graphWithId.node(idStr);
      }).filter(bitId => bitId && bitId.hasScope()).filter(bitId => !_this2.components.find(c => c.id.isEqual(bitId)));

      const scopeComponentsImporter = _scopeComponentsImporter().default.getInstance(_this2.scope);

      yield scopeComponentsImporter.importMany(_bitId().BitIds.fromArray(bitIds));
    })();
  }

  getAllFlattenedDependencies(componentId) {
    var _this3 = this;

    return (0, _bluebird().coroutine)(function* () {
      const {
        graphDeps,
        graphDevDeps,
        graphExtensionDeps
      } = _this3.allDependenciesGraphs;
      const flattenedDependencies = yield _this3.getFlattenedDependencies({
        componentId,
        graph: graphDeps
      });
      const flattenedDevDependencies = yield _this3.getFlattenedDependencies({
        componentId,
        graph: graphDevDeps,
        prodGraph: graphDeps
      });
      const flattenedExtensionDependencies = yield _this3.getFlattenedDependencies({
        componentId,
        graph: graphExtensionDeps,
        prodGraph: graphDeps
      });

      const getFlattenedDevDeps = () => {
        // remove extensions dependencies that are also regular dependencies
        // (no need to do the same for devDependencies, because their duplicated are removed previously)
        const flattenedExt = flattenedExtensionDependencies.removeMultipleIfExistWithoutVersion(flattenedDependencies);
        return _bitId().BitIds.uniqFromArray([...flattenedDevDependencies, ...flattenedExt]);
      };

      return {
        flattenedDependencies,
        flattenedDevDependencies: getFlattenedDevDeps()
      };
    })();
  }

  getFlattenedDependencies({
    componentId,
    graph,
    prodGraph
  }) {
    var _this4 = this;

    return (0, _bluebird().coroutine)(function* () {
      const id = componentId.toString();
      const edges = getEdges(graph, id);
      if (!edges) return new (_bitId().BitIds)();
      const dependencies = getEdgesWithProdGraph(prodGraph, edges);
      if (!dependencies.length) return new (_bitId().BitIds)();

      const flattenDependency = /*#__PURE__*/function () {
        var _ref2 = (0, _bluebird().coroutine)(function* (dependency) {
          if (_this4.cache[dependency]) return _this4.cache[dependency]; // @ts-ignore if graph doesn't have the node, prodGraph must have it

          const dependencyBitId = graph.node(dependency) || prodGraph.node(dependency);
          let versionDependencies;
          if (_this4.notFoundDependencies.has(dependencyBitId)) return [dependencyBitId];

          const scopeComponentsImporter = _scopeComponentsImporter().default.getInstance(_this4.scope);

          try {
            versionDependencies = yield scopeComponentsImporter.importDependencies(_bitId().BitIds.fromArray([dependencyBitId]));
          } catch (err) {
            if (err instanceof _exceptions().DependencyNotFound) {
              _this4.notFoundDependencies.push(dependencyBitId);

              throwWhenDepNotIncluded(componentId, dependencyBitId);
              return [dependencyBitId];
            }

            throw err;
          }

          const flattenedDependencies = yield (0, _flattenDependencies().flattenDependencyIds)(versionDependencies, _this4.scope.objects); // Store the flatten dependencies in cache

          _this4.cache[dependency] = flattenedDependencies;
          return flattenedDependencies;
        });

        return function flattenDependency(_x2) {
          return _ref2.apply(this, arguments);
        };
      }();

      const flattened = yield (0, _bluebird().mapSeries)(dependencies, flattenDependency);

      const flattenedUnique = _bitId().BitIds.uniqFromArray(_ramda().default.flatten(flattened)); // when a component has cycle dependencies, the flattenedDependencies contains the component itself. remove it.


      return flattenedUnique.removeIfExistWithoutVersion(componentId);
    })();
  }

}

exports.FlattenedDependenciesGetter = FlattenedDependenciesGetter;

function throwWhenDepNotIncluded(componentId, dependencyId) {
  if (!dependencyId.hasScope() && !dependencyId.hasVersion()) {
    throw new (_generalError().default)(`fatal: "${componentId.toString()}" has a dependency "${dependencyId.toString()}".
this dependency was not included in the tag command.`);
  }
}

function getEdges(graph, id) {
  if (!graph.hasNode(id)) return null; // @ts-ignore

  const edges = _graphlib().default.alg.preorder(graph, id);

  return _ramda().default.tail(edges); // the first item is the component itself
}
/**
 * for non-prod files, such as test files, we're interested also with its prod dependency.
 * for example, a test file foo.spec.js of component 'foo', requires bar.js from component
 * 'bar'. 'bar.js' requires 'baz.js' from component 'baz'.
 * when calculating the edges of foo.spec.js by devGraph only, we'll get bar.js but not
 * baz.js because the relationship between bar and baz are set on prodGraph only.
 * @see dev-dependencies.e2e, 'dev-dependency that requires prod-dependency' case.
 */


function getEdgesWithProdGraph(prodGraph, dependencies) {
  if (!prodGraph) return dependencies;

  const prodDependencies = _ramda().default.flatten(dependencies.map(dependency => getEdges(prodGraph, dependency))).filter(x => x);

  return _ramda().default.uniq([...dependencies, ...prodDependencies]);
}