"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.promise");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _groupArray() {
  const data = _interopRequireDefault(require("group-array"));

  _groupArray = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _analytics() {
  const data = require("../../analytics/analytics");

  _analytics = function () {
    return data;
  };

  return data;
}

function _bitId() {
  const data = require("../../bit-id");

  _bitId = function () {
    return data;
  };

  return data;
}

function _generalError() {
  const data = _interopRequireDefault(require("../../error/general-error"));

  _generalError = function () {
    return data;
  };

  return data;
}

function _showDoctorError() {
  const data = _interopRequireDefault(require("../../error/show-doctor-error"));

  _showDoctorError = function () {
    return data;
  };

  return data;
}

function _enrichContextFromGlobal() {
  const data = _interopRequireDefault(require("../../hooks/utils/enrich-context-from-global"));

  _enrichContextFromGlobal = function () {
    return data;
  };

  return data;
}

function _logger() {
  const data = _interopRequireDefault(require("../../logger/logger"));

  _logger = function () {
    return data;
  };

  return data;
}

function _utils() {
  const data = require("../../utils");

  _utils = function () {
    return data;
  };

  return data;
}

function _exceptions() {
  const data = require("../exceptions");

  _exceptions = function () {
    return data;
  };

  return data;
}

function _exceptions2() {
  const data = require("../network/exceptions");

  _exceptions2 = function () {
    return data;
  };

  return data;
}

function _objects() {
  const data = require("../objects");

  _objects = function () {
    return data;
  };

  return data;
}

function _scopeRemotes() {
  const data = require("../scope-remotes");

  _scopeRemotes = function () {
    return data;
  };

  return data;
}

function _versionDependencies() {
  const data = _interopRequireDefault(require("../version-dependencies"));

  _versionDependencies = function () {
    return data;
  };

  return data;
}

const removeNils = _ramda().default.reject(_ramda().default.isNil);

class ScopeComponentsImporter {
  constructor(scope) {
    (0, _defineProperty2().default)(this, "scope", void 0);
    (0, _defineProperty2().default)(this, "sources", void 0);
    if (!scope) throw new Error('unable to instantiate ScopeComponentsImporter without Scope');
    this.scope = scope;
    this.sources = scope.sources;
  }

  static getInstance(scope) {
    return new ScopeComponentsImporter(scope);
  }
  /**
   * 1. Local objects, fetch from local. (done by this.sources.getMany method)
   * 2. Fetch flattened dependencies (done by toVersionDependencies method). If they're not locally, fetch from remote
   * and save them locally.
   * 3. External objects, fetch from a remote and save locally. (done by this.getExternalOnes method).
   */


  importMany(ids, cache = true, persist = true) {
    var _this = this;

    return (0, _bluebird().coroutine)(function* () {
      _logger().default.debugAndAddBreadCrumb('ScopeComponentsImporter', 'importMany: {ids}', {
        ids: ids.toString()
      });

      const idsWithoutNils = removeNils(ids);
      if (_ramda().default.isEmpty(idsWithoutNils)) return Promise.resolve([]);

      const [locals, externals] = _ramda().default.partition(id => id.isLocal(_this.scope.name), idsWithoutNils);

      const localDefs = yield _this.sources.getMany(locals);
      const versionDeps = yield (0, _bluebird().mapSeries)(localDefs, def => {
        if (!def.component) throw new (_exceptions().ComponentNotFound)(def.id.toString());
        return _this.componentToVersionDependencies(def.component, def.id);
      });

      _logger().default.debugAndAddBreadCrumb('ScopeComponentsImporter', 'importMany: successfully fetched local components and their dependencies. Going to fetch externals');

      const remotes = yield (0, _scopeRemotes().getScopeRemotes)(_this.scope);
      const externalDeps = yield _this._getExternalMany(externals, remotes, cache, persist);
      return versionDeps.concat(externalDeps);
    })();
  }

  importManyWithoutDependencies(ids, cache = true) {
    var _this2 = this;

    return (0, _bluebird().coroutine)(function* () {
      if (!ids.length) return [];

      _logger().default.debug(`importManyWithoutDependencies. Ids: ${ids.join(', ')}, cache: ${cache.toString()}`);

      _analytics().Analytics.addBreadCrumb('importManyWithoutDependencies', `Ids: ${_analytics().Analytics.hashData(ids)}`);

      const idsWithoutNils = removeNils(ids);
      if (_ramda().default.isEmpty(idsWithoutNils)) return Promise.resolve([]);
      const [externals, locals] = (0, _utils().splitBy)(idsWithoutNils, id => id.isLocal(_this2.scope.name));
      const localDefs = yield _this2.sources.getMany(locals);
      const componentVersionArr = yield Promise.all(localDefs.map(def => {
        if (!def.component) throw new (_exceptions().ComponentNotFound)(def.id.toString()); // $FlowFixMe it must have a version
        // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

        return def.component.toComponentVersion(def.id.version);
      }));
      const remotes = yield (0, _scopeRemotes().getScopeRemotes)(_this2.scope);
      const externalDeps = yield _this2._getExternalManyWithoutDependencies(externals, remotes, cache);
      return componentVersionArr.concat(externalDeps);
    })();
  }
  /**
   * todo: improve performance by finding all versions needed and fetching them in one request from the server
   * currently it goes to the server twice. First, it asks for the last version of each id, and then it goes again to
   * ask for the older versions.
   */


  importManyWithAllVersions(ids, cache = true, allDepsVersions = false // by default, only dependencies of the latest version are imported
  ) {
    var _this3 = this;

    return (0, _bluebird().coroutine)(function* () {
      _logger().default.debug(`scope.getManyWithAllVersions, Ids: ${ids.join(', ')}`);

      _analytics().Analytics.addBreadCrumb('getManyWithAllVersions', `scope.getManyWithAllVersions, Ids: ${_analytics().Analytics.hashData(ids)}`);

      const idsWithoutNils = removeNils(ids);
      if (_ramda().default.isEmpty(idsWithoutNils)) return Promise.resolve([]);
      const versionDependenciesArr = yield _this3.importMany(idsWithoutNils, cache);
      const allIdsWithAllVersions = new (_bitId().BitIds)();
      versionDependenciesArr.forEach(versionDependencies => {
        const versions = versionDependencies.component.component.listVersions();
        const versionId = versionDependencies.component.id;
        const idsWithAllVersions = versions.map(version => {
          if (version === versionDependencies.component.version) return null; // imported already

          return versionId.changeVersion(version);
        });
        allIdsWithAllVersions.push(...removeNils(idsWithAllVersions));
        const head = versionDependencies.component.component.getHead();

        if (head) {
          allIdsWithAllVersions.push(versionId.changeVersion(head.toString()));
        }
      });

      if (allDepsVersions) {
        const verDepsOfOlderVersions = yield _this3.importMany(allIdsWithAllVersions, cache);
        versionDependenciesArr.push(...verDepsOfOlderVersions);
        const allFlattenDepsIds = versionDependenciesArr.map(v => v.allDependencies.map(d => d.id));

        const dependenciesOnly = _ramda().default.flatten(allFlattenDepsIds).filter(id => !ids.hasWithoutVersion(id));

        const verDepsOfAllFlattenDeps = yield _this3.importManyWithAllVersions(_bitId().BitIds.uniqFromArray(dependenciesOnly));
        versionDependenciesArr.push(...verDepsOfAllFlattenDeps);
      } else {
        yield _this3.importManyWithoutDependencies(allIdsWithAllVersions);
      }

      return versionDependenciesArr;
    })();
  }

  importDependencies(dependencies) {
    return new Promise((resolve, reject) => {
      return this.importMany(dependencies).then(resolve).catch(e => {
        _logger().default.error(`importDependencies got an error: ${JSON.stringify(e)}`);

        if (e instanceof _exceptions2().RemoteScopeNotFound || e instanceof _exceptions2().PermissionDenied) return reject(e);
        return reject(new (_exceptions().DependencyNotFound)(e.id));
      });
    });
  }

  importFromLanes(remoteLaneIds) {
    var _this4 = this;

    return (0, _bluebird().coroutine)(function* () {
      const lanes = yield _this4.importLanes(remoteLaneIds);
      const ids = lanes.map(lane => lane.toBitIds());

      const bitIds = _bitId().BitIds.uniqFromArray(_ramda().default.flatten(ids));

      yield _this4.importManyWithAllVersions(bitIds, false);
      return lanes;
    })();
  }

  importLanes(remoteLaneIds) {
    var _this5 = this;

    return (0, _bluebird().coroutine)(function* () {
      const remotes = yield (0, _scopeRemotes().getScopeRemotes)(_this5.scope);
      const objectList = yield remotes.fetch(groupByScopeName(remoteLaneIds), _this5.scope, {
        type: 'lane'
      });
      const bitObjects = yield objectList.toBitObjects();
      const lanes = bitObjects.getLanes();
      yield Promise.all(lanes.map(lane => _this5.scope.objects.remoteLanes.syncWithLaneObject(lane.scope, lane)));
      return lanes;
    })();
  }

  componentToVersionDependencies(component, id) {
    var _this6 = this;

    return (0, _bluebird().coroutine)(function* () {
      // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
      const versionComp = component.toComponentVersion(id.version); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

      const source = id.scope;
      const version = yield versionComp.getVersion(_this6.scope.objects);

      if (!version) {
        _logger().default.debug(`toVersionDependencies, component ${component.id().toString()}, version ${versionComp.version} not found, going to fetch from a remote`);

        if (component.scope === _this6.scope.name) {
          // it should have been fetched locally, since it wasn't found, this is an error
          throw new (_showDoctorError().default)(`Version ${versionComp.version} of ${component.id().toString()} was not found in scope ${_this6.scope.name}`);
        }

        return (0, _scopeRemotes().getScopeRemotes)(_this6.scope).then(remotes => {
          return _this6._getExternal({
            id,
            remotes,
            localFetch: false
          });
        });
      }

      _logger().default.debug(`toVersionDependencies, component ${component.id().toString()}, version ${versionComp.version} found, going to collect its dependencies`);

      const dependencies = yield _this6.importManyWithoutDependencies(version.flattenedDependencies);
      const devDependencies = yield _this6.importManyWithoutDependencies(version.flattenedDevDependencies);
      const extensionsDependencies = yield _this6.importManyWithoutDependencies(version.extensions.extensionsBitIds);
      return new (_versionDependencies().default)(versionComp, dependencies, devDependencies, extensionsDependencies, source);
    })();
  }

  componentsToComponentsObjects(components, clientVersion, collectParents, collectArtifacts) {
    var _this7 = this;

    return (0, _bluebird().coroutine)(function* () {
      const allObject = yield (0, _bluebird().mapSeries)(components, component => component.toObjects(_this7.scope.objects, clientVersion, collectParents, collectArtifacts));
      return _ramda().default.flatten(allObject);
    })();
  }
  /**
   * get ConsumerComponent by bitId. if the component was not found locally, import it from a remote scope
   */


  loadRemoteComponent(id) {
    return this._getComponentVersion(id).then(component => {
      if (!component) throw new (_exceptions().ComponentNotFound)(id.toString()); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

      return component.toConsumer(this.scope.objects);
    });
  }

  loadComponent(id, localOnly = true) {
    _logger().default.debugAndAddBreadCrumb('ScopeComponentsImporter', 'loadComponent {id}', {
      id: id.toString()
    });

    if (localOnly && !id.isLocal(this.scope.name)) {
      throw new (_generalError().default)('cannot load a component from remote scope, please import first');
    }

    return this.loadRemoteComponent(id);
  }
  /**
   * recursive function.
   * if found locally, use them. Otherwise, fetch from remote and then, save into the model.
   */


  _getExternalMany(ids, remotes, localFetch = true, persist = true, context = {}) {
    var _this8 = this;

    return (0, _bluebird().coroutine)(function* () {
      if (!ids.length) return Promise.resolve([]);

      _logger().default.debugAndAddBreadCrumb('scope.getExternalMan', `planning on fetching from ${localFetch ? 'local' : 'remote'} scope. Ids: {ids}`, {
        ids: ids.join(', ')
      });

      (0, _enrichContextFromGlobal().default)(Object.assign(context, {
        requestedBitIds: ids.map(id => id.toString())
      }));
      const componentDefs = yield _this8.sources.getMany(ids);
      const left = componentDefs.filter(componentDef => {
        if (!localFetch) return true;
        if (!componentDef.component) return true;
        return false;
      });

      if (left.length === 0) {
        _logger().default.debugAndAddBreadCrumb('scope.getExternalMany', 'no more ids left, all found locally, exiting the method');

        return (0, _bluebird().mapSeries)(componentDefs, compDef => _this8.componentToVersionDependencies(compDef.component, compDef.id));
      }

      _logger().default.debugAndAddBreadCrumb('scope.getExternalMany', `${left.length} left. Fetching them from a remote`);

      const objectList = yield remotes.fetch(groupByScopeName(left.map(defLeft => defLeft.id)), _this8.scope, undefined, context);

      _logger().default.debugAndAddBreadCrumb('scope.getExternalMany', 'writing them to the model');

      const nonLaneIds = yield _this8.scope.writeManyComponentsToModel(objectList, persist, ids);
      return _this8._getExternalMany(nonLaneIds, remotes);
    })();
  }
  /**
   * If the component is not in the local scope, fetch it from a remote and save into the local
   * scope. (objects directory).
   */


  _getExternal({
    id,
    remotes,
    localFetch = true,
    context = {}
  }) {
    var _this9 = this;

    return (0, _bluebird().coroutine)(function* () {
      (0, _enrichContextFromGlobal().default)(context);
      const component = yield _this9.sources.get(id);

      if (component && localFetch) {
        return _this9.componentToVersionDependencies(component, id);
      }

      const objectList = yield remotes.fetch(groupByScopeName([id]), _this9.scope, undefined, context);
      yield _this9.scope.writeManyComponentsToModel(objectList, true, [id]);
      return _this9._getExternal({
        id,
        remotes,
        localFetch: true
      });
    })();
  }

  _getExternalWithoutDependencies({
    id,
    remotes,
    localFetch = true,
    context = {}
  }) {
    var _this10 = this;

    return (0, _bluebird().coroutine)(function* () {
      const component = yield _this10.sources.get(id);

      if (component && localFetch) {
        return component.toComponentVersion(id.version);
      }

      const objectList = yield remotes.fetch(groupByScopeName([id]), _this10.scope, {
        withoutDependencies: true
      }, context);
      yield _this10.scope.writeManyComponentsToModel(objectList, true, [id]);
      const versionDependencies = yield _this10._getExternal({
        id,
        remotes,
        localFetch: true
      });
      return versionDependencies.component;
    })();
  }

  _getExternalManyWithoutDependencies(ids, remotes, localFetch = false, context = {}) {
    if (!ids.length) return Promise.resolve([]);

    _logger().default.debugAndAddBreadCrumb('_getExternalManyWithoutDependencies', `ids: {ids}, localFetch: ${localFetch.toString()}`, {
      ids: ids.join(', ')
    });

    (0, _enrichContextFromGlobal().default)(Object.assign(context, {
      requestedBitIds: ids.map(id => id.toString())
    }));
    return this.sources.getMany(ids).then(defs => {
      const left = defs.filter(def => {
        if (!localFetch) return true;
        if (!def.component) return true;
        return false;
      });

      if (left.length === 0) {
        _logger().default.debugAndAddBreadCrumb('_getExternalManyWithoutDependencies', 'no more ids left, all found locally, exiting the method'); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!


        return Promise.all(defs.map(def => def.component.toComponentVersion(def.id.version)));
      }

      _logger().default.debugAndAddBreadCrumb('_getExternalManyWithoutDependencies', `${left.length} left. Fetching them from a remote`);

      return remotes.fetch(groupByScopeName(left.map(def => def.id)), this.scope, {
        withoutDependencies: true
      }, context).then(objectList => {
        return this.scope.writeManyComponentsToModel(objectList, undefined, ids);
      }).then(nonLaneIds => this._getExternalManyWithoutDependencies(nonLaneIds, remotes, true));
    });
  }

  _getComponentVersion(id) {
    var _this11 = this;

    return (0, _bluebird().coroutine)(function* () {
      if (!id.isLocal(_this11.scope.name)) {
        const remotes = yield (0, _scopeRemotes().getScopeRemotes)(_this11.scope);
        return _this11._getExternalWithoutDependencies({
          id,
          remotes,
          localFetch: true
        });
      }

      return _this11.sources.get(id).then(component => {
        if (!component) throw new (_exceptions().ComponentNotFound)(id.toString()); // $FlowFixMe version is set
        // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

        return component.toComponentVersion(id.version);
      });
    })();
  }
  /**
   * currently used for import artifacts, but can be used to import any arbitrary array of hashes.
   * it takes care to remove any duplications and check whether the object exists locally before
   * going to the remote
   */


  importManyObjects(groupedHashes) {
    var _this12 = this;

    return (0, _bluebird().coroutine)(function* () {
      const groupedHashedMissing = {};
      yield Promise.all(Object.keys(groupedHashes).map( /*#__PURE__*/function () {
        var _ref = (0, _bluebird().coroutine)(function* (scopeName) {
          const uniqueHashes = _ramda().default.uniq(groupedHashes[scopeName]);

          const missing = yield (0, _bluebird().filter)(uniqueHashes, /*#__PURE__*/function () {
            var _ref2 = (0, _bluebird().coroutine)(function* (hash) {
              return !(yield _this12.scope.objects.has(new (_objects().Ref)(hash)));
            });

            return function (_x2) {
              return _ref2.apply(this, arguments);
            };
          }());

          if (missing.length) {
            groupedHashedMissing[scopeName] = missing;
          }
        });

        return function (_x) {
          return _ref.apply(this, arguments);
        };
      }()));
      if (_ramda().default.isEmpty(groupedHashedMissing)) return;
      const remotes = yield (0, _scopeRemotes().getScopeRemotes)(_this12.scope);
      const objectList = yield remotes.fetch(groupedHashedMissing, _this12.scope, {
        type: 'object'
      });
      const bitObjectsList = yield objectList.toBitObjects();

      _this12.scope.objects.addMany(bitObjectsList.getAll());

      _this12.scope.objects.persist();
    })();
  }

}

exports.default = ScopeComponentsImporter;

function groupByScopeName(ids) {
  const grouped = (0, _groupArray().default)(ids, 'scope');
  Object.keys(grouped).forEach(scopeName => {
    grouped[scopeName] = grouped[scopeName].map(id => id.toString());
  });
  return grouped;
}