"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.promise");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _fsExtra() {
  const data = _interopRequireDefault(require("fs-extra"));

  _fsExtra = function () {
    return data;
  };

  return data;
}

function _path() {
  const data = _interopRequireDefault(require("path"));

  _path = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _bitId() {
  const data = require("../../bit-id");

  _bitId = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

function _laneId() {
  const data = require("../../lane-id/lane-id");

  _laneId = function () {
    return data;
  };

  return data;
}

function _utils() {
  const data = require("../../utils");

  _utils = function () {
    return data;
  };

  return data;
}

function _objects() {
  const data = require("../objects");

  _objects = function () {
    return data;
  };

  return data;
}

/**
 * each lane holds components and hashes, which are the heads of the remote
 */
class RemoteLanes {
  constructor(scopePath) {
    (0, _defineProperty2().default)(this, "basePath", void 0);
    (0, _defineProperty2().default)(this, "remotes", void 0);
    this.basePath = _path().default.join(scopePath, _constants().REMOTE_REFS_DIR);
    this.remotes = {};
  }

  addEntry(remoteLaneId, componentId, head) {
    var _this = this;

    return (0, _bluebird().coroutine)(function* () {
      if (!remoteLaneId) throw new TypeError('addEntry expects to get remoteLaneId');
      if (!head) return; // do nothing

      const remoteLane = yield _this.getRemoteLane(remoteLaneId);
      const existingComponent = remoteLane.find(n => n.id.isEqualWithoutVersion(componentId));

      if (existingComponent) {
        existingComponent.head = head;
      } else {
        remoteLane.push({
          id: componentId,
          head
        });
      }
    })();
  }

  getRef(remoteLaneId, bitId) {
    var _this2 = this;

    return (0, _bluebird().coroutine)(function* () {
      if (!remoteLaneId) throw new TypeError('getEntry expects to get remoteLaneId');

      if (!_this2.remotes[remoteLaneId.scope] || !_this2.remotes[remoteLaneId.scope][remoteLaneId.name]) {
        yield _this2.loadRemoteLane(remoteLaneId);
      }

      const remoteLane = _this2.remotes[remoteLaneId.scope][remoteLaneId.name];
      const existingComponent = remoteLane.find(n => n.id.isEqualWithoutVersion(bitId));
      if (!existingComponent) return null;
      return existingComponent.head;
    })();
  }

  getRemoteLane(remoteLaneId) {
    var _this3 = this;

    return (0, _bluebird().coroutine)(function* () {
      if (!_this3.remotes[remoteLaneId.scope] || !_this3.remotes[remoteLaneId.scope][remoteLaneId.name]) {
        yield _this3.loadRemoteLane(remoteLaneId);
      }

      return _this3.remotes[remoteLaneId.scope][remoteLaneId.name];
    })();
  }

  getRemoteBitIds(remoteLaneId) {
    var _this4 = this;

    return (0, _bluebird().coroutine)(function* () {
      const remoteLane = yield _this4.getRemoteLane(remoteLaneId);
      return remoteLane.map(item => item.id.changeVersion(item.head.toString()));
    })();
  }

  loadRemoteLane(remoteLaneId) {
    var _this5 = this;

    return (0, _bluebird().coroutine)(function* () {
      const remoteName = remoteLaneId.scope;
      const laneName = remoteLaneId.name;

      const remoteLanePath = _this5.composeRemoteLanePath(remoteName, laneName);

      try {
        const remoteFile = yield _fsExtra().default.readJson(remoteLanePath);
        if (!_this5.remotes[remoteName]) _this5.remotes[remoteName] = {};
        _this5.remotes[remoteName][laneName] = remoteFile.map(({
          id,
          head
        }) => ({
          id: new (_bitId().BitId)({
            scope: id.scope,
            name: id.name
          }),
          head: new (_objects().Ref)(head)
        }));
      } catch (err) {
        if (err.code === 'ENOENT') {
          if (!_this5.remotes[remoteName]) _this5.remotes[remoteName] = {};
          _this5.remotes[remoteName][laneName] = [];
          return;
        }

        throw err;
      }
    })();
  }

  getAllRemoteLaneIds() {
    var _this6 = this;

    return (0, _bluebird().coroutine)(function* () {
      const matches = yield (0, _utils().glob)(_path().default.join('*', '*'), {
        cwd: _this6.basePath
      });
      return matches.map(match => {
        const split = match.split(_path().default.sep); // in the future, lane-name might have slashes, so until the first slash is the scope.
        // the rest are the name

        return _laneId().RemoteLaneId.from(_ramda().default.tail(split).join('/'), _ramda().default.head(split));
      });
    })();
  }

  syncWithLaneObject(remoteName, lane) {
    var _this7 = this;

    return (0, _bluebird().coroutine)(function* () {
      const remoteLaneId = _laneId().RemoteLaneId.from(lane.name, remoteName);

      if (!_this7.remotes[remoteName] || !_this7.remotes[remoteName][lane.name]) {
        yield _this7.loadRemoteLane(remoteLaneId);
      }

      yield Promise.all(lane.components.map(component => _this7.addEntry(remoteLaneId, component.id, component.head)));
    })();
  }

  composeRemoteLanePath(remoteName, laneName) {
    return _path().default.join(this.basePath, remoteName, laneName);
  }

  write() {
    var _this8 = this;

    return (0, _bluebird().coroutine)(function* () {
      return Promise.all(Object.keys(_this8.remotes).map(remoteName => _this8.writeRemoteLanes(remoteName)));
    })();
  }

  writeRemoteLanes(remoteName) {
    var _this9 = this;

    return (0, _bluebird().coroutine)(function* () {
      return Promise.all(Object.keys(_this9.remotes[remoteName]).map(laneName => _this9.writeRemoteLaneFile(remoteName, laneName)));
    })();
  }

  writeRemoteLaneFile(remoteName, laneName) {
    var _this10 = this;

    return (0, _bluebird().coroutine)(function* () {
      const obj = _this10.remotes[remoteName][laneName].map(({
        id,
        head
      }) => ({
        id: {
          scope: id.scope,
          name: id.name
        },
        head: head.toString()
      }));

      return _fsExtra().default.outputFile(_this10.composeRemoteLanePath(remoteName, laneName), JSON.stringify(obj, null, 2));
    })();
  }

}

exports.default = RemoteLanes;