"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _fsExtra() {
  const data = _interopRequireDefault(require("fs-extra"));

  _fsExtra = function () {
    return data;
  };

  return data;
}

function _path() {
  const data = _interopRequireDefault(require("path"));

  _path = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _objects() {
  const data = require("../objects");

  _objects = function () {
    return data;
  };

  return data;
}

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2().default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

const UNMERGED_FILENAME = 'unmerged.json';

class UnmergedComponents {
  constructor(filePath, unmerged) {
    (0, _defineProperty2().default)(this, "filePath", void 0);
    (0, _defineProperty2().default)(this, "unmerged", void 0);
    (0, _defineProperty2().default)(this, "hasChanged", false);
    this.filePath = filePath;
    this.unmerged = unmerged;
  }

  static load(scopePath) {
    return (0, _bluebird().coroutine)(function* () {
      const filePath = _path().default.join(scopePath, UNMERGED_FILENAME);

      let unmerged;

      try {
        const fileContent = yield _fsExtra().default.readJson(filePath);
        unmerged = fileContent.map(item => _objectSpread(_objectSpread({}, item), {}, {
          head: _objects().Ref.from(item.head)
        }));
      } catch (err) {
        if (err.code === 'ENOENT') {
          unmerged = [];
        } else {
          throw err;
        }
      }

      return new UnmergedComponents(filePath, unmerged);
    })();
  }

  addEntry(unmergedComponent) {
    const existingComponent = this.getEntry(unmergedComponent.id.name);

    if (existingComponent) {
      throw new Error(`unable to add an unmerged component, a component with the same name already exist. name ${existingComponent.id.name}, scope: ${existingComponent.id.scope}`);
    }

    this.unmerged.push(unmergedComponent);
    this.hasChanged = true;
  }

  getEntry(componentName) {
    return this.unmerged.find(u => u.id.name === componentName);
  }

  getResolvedComponents() {
    return this.unmerged.filter(u => u.resolved);
  }

  getUnresolvedComponents() {
    return this.unmerged.filter(u => !u.resolved);
  }

  getComponents() {
    return this.unmerged;
  }

  removeComponent(componentName) {
    const found = this.getEntry(componentName);
    if (!found) return false;
    this.unmerged = _ramda().default.without([found], this.unmerged);
    this.hasChanged = true;
    return true;
  }

  write() {
    var _this = this;

    return (0, _bluebird().coroutine)(function* () {
      if (!_this.hasChanged) return;
      yield _fsExtra().default.outputFile(_this.filePath, JSON.stringify(_this.toObject(), null, 2));
    })();
  }

  toObject() {
    return this.unmerged.map(item => _objectSpread(_objectSpread({}, item), {}, {
      head: item.head.toString()
    }));
  }

  static buildSnapMessage(unmergedComponent) {
    const remote = unmergedComponent.remote ? `remote ${unmergedComponent.remote}/` : '';
    return `merge ${remote}${unmergedComponent.lane}`;
  }

}

exports.default = UnmergedComponents;