/// <reference types="node" />
import { BitId, BitIds } from '../../bit-id';
import ConsumerComponent from '../../consumer/component';
import { CustomResolvedPath } from '../../consumer/component/consumer-component';
import { Dependencies, Dependency } from '../../consumer/component/dependencies';
import { SourceFile } from '../../consumer/component/sources';
import { ComponentOverridesData } from '../../consumer/config/component-overrides';
import { ExtensionDataList } from '../../consumer/config/extension-data';
import { Results } from '../../consumer/specs-results/specs-results';
import { Doclet } from '../../jsdoc/types';
import { CompilerExtensionModel } from '../../legacy-extensions/compiler-extension';
import { EnvPackages } from '../../legacy-extensions/env-extension';
import { TesterExtensionModel } from '../../legacy-extensions/tester-extension';
import { PathLinux, PathLinuxRelative } from '../../utils/path';
import { BitObject, Ref } from '../objects';
import { ObjectItem } from '../objects/object-list';
import Repository from '../objects/repository';
import Source from './source';
declare type CiProps = {
    error: Record<string, any>;
    startTime: string;
    endTime: string;
};
export declare type SourceFileModel = {
    name: string;
    relativePath: PathLinux;
    test: boolean;
    file: Ref;
};
export declare type DistFileModel = SourceFileModel;
export declare type ArtifactFileModel = {
    relativePath: PathLinux;
    file: Ref;
};
export declare type Log = {
    message: string;
    date: string;
    username: string | undefined;
    email: string | undefined;
};
export declare type VersionProps = {
    mainFile: PathLinux;
    files: Array<SourceFileModel>;
    dists?: Array<DistFileModel> | undefined;
    mainDistFile: PathLinux | undefined;
    compiler?: CompilerExtensionModel | undefined;
    tester?: TesterExtensionModel | undefined;
    log: Log;
    ci?: CiProps;
    specsResults?: Results | undefined;
    docs?: Doclet[];
    dependencies?: Dependency[];
    devDependencies?: Dependency[];
    flattenedDependencies?: BitIds;
    flattenedDevDependencies?: BitIds;
    packageDependencies?: {
        [key: string]: string;
    };
    devPackageDependencies?: {
        [key: string]: string;
    };
    peerPackageDependencies?: {
        [key: string]: string;
    };
    compilerPackageDependencies?: EnvPackages;
    testerPackageDependencies?: EnvPackages;
    bindingPrefix?: string;
    schema?: string;
    customResolvedPaths?: CustomResolvedPath[];
    overrides: ComponentOverridesData;
    packageJsonChangedProps?: Record<string, any>;
    hash?: string;
    parents?: Ref[];
    extensions?: ExtensionDataList;
};
export default class Version extends BitObject {
    mainFile: PathLinux;
    files: Array<SourceFileModel>;
    dists: Array<DistFileModel> | undefined;
    mainDistFile: PathLinuxRelative | undefined;
    compiler: CompilerExtensionModel | undefined;
    tester: TesterExtensionModel | undefined;
    log: Log;
    ci: CiProps | {};
    specsResults: Results | undefined;
    docs: Doclet[] | undefined;
    dependencies: Dependencies;
    devDependencies: Dependencies;
    flattenedDependencies: BitIds;
    flattenedDevDependencies: BitIds;
    packageDependencies: {
        [key: string]: string;
    };
    devPackageDependencies: {
        [key: string]: string;
    };
    peerPackageDependencies: {
        [key: string]: string;
    };
    compilerPackageDependencies: EnvPackages;
    testerPackageDependencies: EnvPackages;
    bindingPrefix: string | undefined;
    schema: string | undefined;
    customResolvedPaths: CustomResolvedPath[] | undefined;
    overrides: ComponentOverridesData;
    packageJsonChangedProps: Record<string, any>;
    _hash: string;
    parents: Ref[];
    extensions: ExtensionDataList;
    constructor(props: VersionProps);
    validateVersion(): void;
    id(): string;
    calculateHash(): Ref;
    hash(): Ref;
    get extensionDependencies(): Dependencies;
    getAllFlattenedDependencies(): BitIds;
    getAllDependencies(): Dependency[];
    get depsIdsGroupedByType(): {
        dependencies: BitIds;
        devDependencies: BitIds;
        extensionDependencies: BitIds;
    };
    getAllDependenciesCloned(): Dependencies;
    getAllDependenciesIds(): BitIds;
    getDependenciesIdsExcludeExtensions(): BitIds;
    updateFlattenedDependency(currentId: BitId, newId: BitId): void;
    refs(): Ref[];
    refsWithOptions(includeParents?: boolean, includeArtifacts?: boolean): Ref[];
    refsWithoutParents(): Ref[];
    collectManyObjects(repo: Repository, refs: Ref[]): Promise<ObjectItem[]>;
    toObject(): any;
    validateBeforePersisting(versionStr: string): void;
    toBuffer(pretty: boolean): Buffer;
    static parse(contents: string, hash: string): Version;
    static from(versionProps: VersionProps, hash: string): Version;
    static fromComponent({ component, files, dists, mainDistFile, }: {
        component: ConsumerComponent;
        files: Array<SourceFileModel>;
        dists: Array<DistFileModel> | undefined;
        mainDistFile: PathLinuxRelative;
    }): Version;
    setNewHash(): void;
    get ignoreSharedDir(): boolean;
    get isLegacy(): boolean;
    setSpecsResults(specsResults: Results | undefined): void;
    setDist(dist: Source | undefined): void;
    setCIProps(ci: CiProps): void;
    hasParent(ref: Ref): Ref | undefined;
    addParent(ref: Ref): void;
    addAsOnlyParent(ref: Ref): void;
    removeParent(ref: Ref): void;
    modelFilesToSourceFiles(repository: Repository): Promise<SourceFile[]>;
    validate(): void;
}
export {};
