"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Http = void 0;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _graphqlRequest() {
  const data = require("graphql-request");

  _graphqlRequest = function () {
    return data;
  };

  return data;
}

function _nodeFetch() {
  const data = _interopRequireDefault(require("node-fetch"));

  _nodeFetch = function () {
    return data;
  };

  return data;
}

function _bitId() {
  const data = require("../../../bit-id");

  _bitId = function () {
    return data;
  };

  return data;
}

function _component() {
  const data = _interopRequireDefault(require("../../../consumer/component"));

  _component = function () {
    return data;
  };

  return data;
}

function _globalFlags() {
  const data = _interopRequireDefault(require("../../../cli/global-flags"));

  _globalFlags = function () {
    return data;
  };

  return data;
}

function _globalConfig() {
  const data = require("../../../api/consumer/lib/global-config");

  _globalConfig = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

function _logger() {
  const data = _interopRequireDefault(require("../../../logger/logger"));

  _logger = function () {
    return data;
  };

  return data;
}

function _objectList() {
  const data = require("../../objects/object-list");

  _objectList = function () {
    return data;
  };

  return data;
}

function _remoteErrorHandler() {
  const data = require("../remote-error-handler");

  _remoteErrorHandler = function () {
    return data;
  };

  return data;
}

class Http {
  constructor(scopeUrl) {
    this.scopeUrl = scopeUrl;
    (0, _defineProperty2().default)(this, "_token", void 0);
  }

  get token() {
    if (this._token === undefined) return this._token;

    const processToken = _globalFlags().default.token;

    const token = processToken || (0, _globalConfig().getSync)(_constants().CFG_USER_TOKEN_KEY);
    if (!token) this._token = null;
    return token;
  }

  get graphqlUrl() {
    return `${this.scopeUrl}/graphql`;
  }

  close() {}

  describeScope() {
    var _this = this;

    return (0, _bluebird().coroutine)(function* () {
      const SCOPE_QUERY = (0, _graphqlRequest().gql)`
      {
        scope {
          name
        }
      }
    `;
      const data = yield (0, _graphqlRequest().request)(_this.graphqlUrl, SCOPE_QUERY);
      return {
        name: data.scope.name
      };
    })();
  }

  deleteMany(ids, force, context, idsAreLanes) {
    var _this2 = this;

    return (0, _bluebird().coroutine)(function* () {
      const REMOVE_COMPONENTS = (0, _graphqlRequest().gql)`
      query removeComponents($ids: [String], $force: Boolean, $lanes: Boolean) {
        remove(ids: $ids, force: $force, isLanes: $lanes)
      }
    `;
      const res = yield (0, _graphqlRequest().request)(_this2.graphqlUrl, REMOVE_COMPONENTS, {
        ids,
        force,
        idsAreLanes
      });
      return res.removeComponents;
    })();
  }

  pushMany(objectList) {
    var _this3 = this;

    return (0, _bluebird().coroutine)(function* () {
      const route = 'api/scope/put';

      _logger().default.debug(`Http.pushMany, total objects ${objectList.count()}`);

      const pack = objectList.toTar();
      const res = yield (0, _nodeFetch().default)(`${_this3.scopeUrl}/${route}`, {
        method: 'POST',
        body: pack
      });
      yield _this3.throwForNonOkStatus(res);
      const ids = yield res.json();
      return ids;
    })();
  }

  fetch(ids, fetchOptions) {
    var _this4 = this;

    return (0, _bluebird().coroutine)(function* () {
      const route = 'api/scope/fetch';
      const body = JSON.stringify({
        ids,
        fetchOptions
      });
      const res = yield (0, _nodeFetch().default)(`${_this4.scopeUrl}/${route}`, {
        method: 'post',
        body,
        headers: _this4.getHeaders({
          'Content-Type': 'application/json'
        })
      });
      yield _this4.throwForNonOkStatus(res);
      const objectList = yield _objectList().ObjectList.fromTar(res.body);
      return objectList;
    })();
  }

  throwForNonOkStatus(res) {
    return (0, _bluebird().coroutine)(function* () {
      if (!res.ok) {
        var _response$error;

        const response = yield res.json();

        _logger().default.error(`parsed error from HTTP, url: ${res.url}`, response);

        const error = ((_response$error = response.error) === null || _response$error === void 0 ? void 0 : _response$error.code) ? response.error : response;
        const err = (0, _remoteErrorHandler().remoteErrorHandler)(error.code, error, res.url, `status: ${res.status}. text: ${res.statusText}`);
        throw err;
      }
    })();
  }

  getHeaders(headers = {}) {
    return Object.assign(headers, {
      Authorization: `Bearer ${this.token}`
    });
  }

  list(namespacesUsingWildcards) {
    var _this5 = this;

    return (0, _bluebird().coroutine)(function* () {
      const LIST_LEGACY = (0, _graphqlRequest().gql)`
      query listLegacy($namespaces: String) {
        scope {
          _legacyList(namespaces: $namespaces) {
            id
            deprecated
          }
        }
      }
    `;
      const data = yield (0, _graphqlRequest().request)(_this5.graphqlUrl, LIST_LEGACY, {
        namespaces: namespacesUsingWildcards
      });

      data.scope._legacyList.forEach(comp => {
        comp.id = _bitId().BitId.parse(comp.id);
      });

      return data.scope._legacyList;
    })();
  }

  show(bitId) {
    var _this6 = this;

    return (0, _bluebird().coroutine)(function* () {
      const SHOW_COMPONENT = (0, _graphqlRequest().gql)`
      query showLegacy($id: String!) {
        scope {
          _getLegacy(id: $id)
        }
      }
    `;
      const data = yield (0, _graphqlRequest().request)(_this6.graphqlUrl, SHOW_COMPONENT, {
        id: bitId.toString()
      });
      return _component().default.fromString(data.scope._getLegacy);
    })();
  }

  deprecateMany(ids) {
    var _this7 = this;

    return (0, _bluebird().coroutine)(function* () {
      const DEPRECATE_COMPONENTS = (0, _graphqlRequest().gql)`
      mutation deprecate($bitIds: [String!]!) {
        deprecate(bitIds: $bitIds)
      }
    `;
      const res = yield (0, _graphqlRequest().request)(_this7.graphqlUrl, DEPRECATE_COMPONENTS, {
        ids
      });
      return res;
    })();
  }

  undeprecateMany(ids) {
    var _this8 = this;

    return (0, _bluebird().coroutine)(function* () {
      const UNDEPRECATE_COMPONENTS = (0, _graphqlRequest().gql)`
      mutation deprecate($bitIds: [String!]!) {
        undeprecate(bitIds: $bitIds)
      }
    `;
      const res = yield (0, _graphqlRequest().request)(_this8.graphqlUrl, UNDEPRECATE_COMPONENTS, {
        ids
      });
      return res;
    })();
  } // TODO: @david please fix this.


  log(id) {
    var _this9 = this;

    return (0, _bluebird().coroutine)(function* () {
      const GET_LOG_QUERY = (0, _graphqlRequest().gql)`
      query getLogs($id: String!) {
        scope {
          getLogs(id: $id) {
            message
            hash
            date
          }
        }
      }
    `;
      const data = yield (0, _graphqlRequest().request)(_this9.graphqlUrl, GET_LOG_QUERY, {
        id: id.toString()
      });
      return data.scope.getLogs;
    })();
  }

  latestVersions(bitIds) {
    var _this10 = this;

    return (0, _bluebird().coroutine)(function* () {
      const GET_LATEST_VERSIONS = (0, _graphqlRequest().gql)`
      query getLatestVersions($ids: [String]!) {
        scope {
          _legacyLatestVersions(ids: $ids)
        }
      }
    `;
      const data = yield (0, _graphqlRequest().request)(_this10.graphqlUrl, GET_LATEST_VERSIONS, {
        ids: bitIds.map(id => id.toString())
      });
      return data.scope._legacyLatestVersions;
    })();
  }

  graph() {
    throw new Error('Method not implemented.');
  } // TODO: ran (TBD)


  listLanes(name, mergeData) {
    var _this11 = this;

    return (0, _bluebird().coroutine)(function* () {
      const LIST_LANES = (0, _graphqlRequest().gql)`
    query listLanes() {
      lanes {
        list()
      }
    }
    `;
      const res = yield (0, _graphqlRequest().request)(_this11.graphqlUrl, LIST_LANES, {
        mergeData
      });
      return res.lanes.list;
    })();
  }

  static connect(host) {
    return (0, _bluebird().coroutine)(function* () {
      return new Http(host);
    })();
  }

}

exports.Http = Http;