"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.promise");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ObjectList = exports.FETCH_FORMAT_OBJECT_LIST = void 0;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _tarStream() {
  const data = _interopRequireDefault(require("tar-stream"));

  _tarStream = function () {
    return data;
  };

  return data;
}

function _() {
  const data = require(".");

  _ = function () {
    return data;
  };

  return data;
}

function _bitObjectList() {
  const data = require("./bit-object-list");

  _bitObjectList = function () {
    return data;
  };

  return data;
}

function _ref() {
  const data = _interopRequireDefault(require("./ref"));

  _ref = function () {
    return data;
  };

  return data;
}

const FETCH_FORMAT_OBJECT_LIST = 'ObjectList';
exports.FETCH_FORMAT_OBJECT_LIST = FETCH_FORMAT_OBJECT_LIST;

class ObjectList {
  constructor(objects = []) {
    this.objects = objects;
  }

  count() {
    return this.objects.length;
  }

  static mergeMultipleInstances(ObjectLists) {
    const objectList = new ObjectList();
    ObjectLists.forEach(objList => objectList.addIfNotExist(objList.objects));
    return objectList;
  }

  static fromJsonString(jsonStr) {
    const jsonParsed = JSON.parse(jsonStr);

    if (!Array.isArray(jsonParsed)) {
      throw new Error(`fromJsonString expect an array, got ${typeof jsonParsed}`);
    }

    jsonParsed.forEach(obj => {
      obj.ref = new (_ref().default)(obj.ref.hash);
      obj.buffer = Buffer.from(obj.buffer);
    });
    return new ObjectList(jsonParsed);
  }

  toJsonString() {
    return JSON.stringify(this.objects);
  }

  toTar() {
    const pack = _tarStream().default.pack();

    this.objects.forEach(obj => {
      pack.entry({
        name: obj.ref.hash
      }, obj.buffer);
    });
    pack.finalize();
    return pack;
  }

  static fromTar(packStream) {
    return (0, _bluebird().coroutine)(function* () {
      const extract = _tarStream().default.extract();

      const objectItems = yield new Promise((resolve, reject) => {
        const objects = [];
        extract.on('entry', (header, stream, next) => {
          let data = Buffer.from('');
          stream.on('data', chunk => {
            data = Buffer.concat([data, chunk]);
          });
          stream.on('end', () => {
            objects.push({
              ref: new (_ref().default)(header.name),
              buffer: data
            });
            next(); // ready for next entry
          });
          stream.on('error', err => reject(err));
          stream.resume(); // just auto drain the stream
        });
        extract.on('finish', () => {
          resolve(objects);
        });
        packStream.pipe(extract);
      });
      return new ObjectList(objectItems);
    })();
  }

  addIfNotExist(objectItems) {
    objectItems.forEach(objectItem => {
      if (!this.objects.find(object => object.ref.isEqual(objectItem.ref))) {
        this.objects.push(objectItem);
      }
    });
  }

  toBitObjects() {
    var _this = this;

    return (0, _bluebird().coroutine)(function* () {
      const bitObjects = yield Promise.all(_this.objects.map(object => _().BitObject.parseObject(object.buffer)));
      return new (_bitObjectList().BitObjectList)(bitObjects);
    })();
  }

}

exports.ObjectList = ObjectList;