import { BitId, BitIds } from '../../bit-id';
import ConsumerComponent from '../../consumer/component';
import Consumer from '../../consumer/consumer';
import ComponentObjects from '../component-objects';
import ComponentNeedsUpdate from '../exceptions/component-needs-update';
import { ModelComponent, Symlink, Version } from '../models';
import Lane from '../models/lane';
import { ComponentProps } from '../models/model-component';
import { BitObject } from '../objects';
import Repository from '../objects/repository';
import Scope from '../scope';
export declare type ComponentTree = {
    component: ModelComponent;
    objects: BitObject[];
};
export declare type LaneTree = {
    lane: Lane;
    objects: BitObject[];
};
export declare type ComponentDef = {
    id: BitId;
    component: ModelComponent | null | undefined;
};
export default class SourceRepository {
    scope: Scope;
    constructor(scope: Scope);
    objects(): Repository;
    getMany(ids: BitId[] | BitIds): Promise<ComponentDef[]>;
    get(bitId: BitId): Promise<ModelComponent | undefined>;
    _findComponent(component: ModelComponent): Promise<ModelComponent | undefined>;
    _findComponentBySymlink(symlink: Symlink): Promise<ModelComponent | undefined>;
    getObjects(id: BitId): Promise<ComponentObjects>;
    findOrAddComponent(props: ComponentProps): Promise<ModelComponent>;
    modifyCIProps({ source, ciProps }: {
        source: ConsumerComponent;
        ciProps: Record<string, any>;
    }): Promise<any>;
    modifySpecsResults({ source, specsResults }: {
        source: ConsumerComponent;
        specsResults?: any;
    }): Promise<any>;
    updateDist({ source }: {
        source: ConsumerComponent;
    }): Promise<any>;
    private transformArtifactsFromVinylToSource;
    consumerComponentToVersion({ consumerComponent, consumer, }: {
        readonly consumerComponent: ConsumerComponent;
        consumer: Consumer;
        force?: boolean;
        verbose?: boolean;
    }): Promise<{
        version: Version;
        files: any;
        dists: any;
        compilerFiles: any;
        testerFiles: any;
    }>;
    enrichSource(consumerComponent: ConsumerComponent): Promise<ConsumerComponent>;
    addSource({ source, consumer, lane, resolveUnmerged, }: {
        source: ConsumerComponent;
        consumer: Consumer;
        lane: Lane | null;
        resolveUnmerged?: boolean;
    }): Promise<ModelComponent>;
    putModelComponent(component: ModelComponent): void;
    put({ component, objects }: ComponentTree): ModelComponent;
    putObjects(objects: BitObject[]): void;
    removeComponentVersions(component: ModelComponent, versions: string[], allVersionsObjects: Version[]): void;
    removeComponentById(bitId: BitId): Promise<void>;
    removeComponent(component: ModelComponent): void;
    mergeTwoComponentsObjects(existingComponent: ModelComponent, incomingComponent: ModelComponent, existingComponentTagsAndSnaps: string[], incomingComponentTagsAndSnaps: string[]): {
        mergedComponent: ModelComponent;
        mergedVersions: string[];
    };
    merge(component: ModelComponent, versionObjects: Version[], inScope?: boolean, local?: boolean): Promise<{
        mergedComponent: ModelComponent;
        mergedVersions: string[];
    }>;
    mergeLane(lane: Lane, local: boolean): Promise<Array<{
        mergedComponent: ModelComponent;
        mergedVersions: string[];
    } | ComponentNeedsUpdate>>;
}
